# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import logging.config
import yaml

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('superdesk')

# set default levels
logging.getLogger('ldap3').setLevel(logging.WARNING)
logging.getLogger('kombu').setLevel(logging.WARNING)
logging.getLogger('elasticsearch').setLevel(logging.ERROR)

logging.getLogger('apps').setLevel(logging.INFO)
logging.getLogger('superdesk').setLevel(logging.INFO)
logging.getLogger('superdesk.websockets_comms').setLevel(logging.WARNING)


def item_msg(msg, item):
    """Return a message with item id appended.

    :param msg: Original message
    :param item: Item object
    """
    return '{} item={}'.format(msg, str(item.get('_id', item.get('guid'))))


def configure_logging(file_path):
    """
    Configure logging.

    :param str file_path:
    """
    if not file_path:
        return

    try:
        with open(file_path, 'r') as f:
            logging_dict = yaml.load(f)

        logging.config.dictConfig(logging_dict)
    except Exception:
        logger.warn('Cannot load logging config. File: %s', file_path)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.metadata.item import metadata_schema, not_analyzed
from content_api import MONGO_PREFIX, ELASTIC_PREFIX

code_mapping = {
    'type': 'object',
    'properties': {
        'name': not_analyzed,
        'code': not_analyzed
    }
}

schema = {
    '_id': metadata_schema['_id'],
    'associations': metadata_schema['associations'],
    'body_html': {'type': 'string'},
    'body_text': {'type': 'string'},
    'byline': {'type': 'string'},
    'copyrightnotice': metadata_schema['copyrightnotice'],
    'copyrightholder': {'type': 'string'},
    'description_html': {'type': 'string'},
    'description_text': {'type': 'string'},
    'headline': {'type': 'string'},
    'language': metadata_schema['language'],
    'located': {'type': 'string'},
    'mimetype': metadata_schema['mimetype'],
    'organization': {'type': 'list'},
    'person': {'type': 'list'},
    'place': metadata_schema['place'],
    'profile': {'type': 'string'},
    'pubstatus': metadata_schema['pubstatus'],
    'renditions': {'type': 'dict'},
    'service': {'type': 'list', 'mapping': code_mapping},
    'slugline': {'type': 'string'},
    'source': metadata_schema['source'],
    'subject': {'type': 'list', 'mapping': code_mapping},
    'keywords': metadata_schema['keywords'],
    'type': metadata_schema['type'],
    'urgency': {'type': 'integer'},
    'priority': {'type': 'integer'},
    'uri': metadata_schema['guid'],  # we use guid value for uri, so index it in a same way
    'usageterms': {'type': 'string'},
    'version': {'type': 'string', 'required': True, 'empty': False, 'nullable': False},
    'versioncreated': {'type': 'datetime', 'required': True},
    'firstcreated': {'type': 'datetime'},
    'firstpublished': {'type': 'datetime', 'required': False},
    'evolvedfrom': Resource.not_analyzed_field(),
    'nextversion': Resource.not_analyzed_field(),
    'subscribers': Resource.not_analyzed_field('list'),
    'ednote': {'type': 'string'},
    'signal': {
        'type': 'list',
        'mapping': {
            'type': 'object',
            'properties': {
                'code': not_analyzed,
                'name': not_analyzed,
                'scheme': not_analyzed
            }
        }
    },
    'genre': {'type': 'list', 'mapping': code_mapping},
    'ancestors': Resource.not_analyzed_field('list'),
    'attachments': {
        'type': 'list',
        'schema': {
            'type': 'dict',
        },
    },
    'annotations': {
        'type': 'list',
        'mapping': {
            'type': 'object',
            'properties': {
                'id': not_analyzed,
                'type': not_analyzed,
                'body': not_analyzed
            }
        }
    },
    'bookmarks': Resource.not_analyzed_field('list'),
    'downloads': Resource.not_analyzed_field('list'),  # list of user ids who downloaded this item
    'shares': Resource.not_analyzed_field('list'),  # list of user ids who shared this item
    'prints': Resource.not_analyzed_field('list'),  # list of user ids who printed this item
    'copies': Resource.not_analyzed_field('list'),  # list of user ids who copied this item
    'extra': metadata_schema['extra'],
    'authors': metadata_schema['authors'],
    'wordcount': metadata_schema['word_count'],
    'charcount': {'type': 'integer'},
    'readtime': {'type': 'integer'},

    # newsroom agenda links
    'event_id': metadata_schema['guid'],
    'planning_id': metadata_schema['guid'],
    'coverage_id': metadata_schema['guid'],
    'agenda_id': metadata_schema['guid'],
    'agenda_href': metadata_schema['guid'],
}


class ItemsResource(Resource):
    """A class defining and configuring the /items API endpoint."""

    # Example of an ID of an object in database (whitout quotes):
    #
    #     "tag:example.com,0000:newsml_BRE9A605"
    #     "tag:localhost:2015:f4b35e12-559b-4a2b-b1f2-d5e64048bde8"
    #
    item_url = r'regex("[\w,.:-]+")'
    schema = schema

    datasource = {
        'search_backend': 'elastic',
        'elastic_filter': {"bool": {"must_not": {"term": {"type": "composite"}}}},
        'default_sort': [('_updated', -1)]
    }

    item_methods = ['GET']
    resource_methods = ['GET']
    versioning = True
    mongo_prefix = MONGO_PREFIX
    elastic_prefix = ELASTIC_PREFIX




# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""This module contains tools to help using Elastic Search"""

import logging
import json
from superdesk import app

logger = logging.getLogger(__name__)

REPOS = ['ingest', 'archive', 'published', 'archived']
POST_FILTER_MAP = {
    "type": "type",
    "desk": "task.desk",
    "genre": "genre.name",
    "category": "anpa_category.name",
    "urgency": "urgency",
    "priority": "priority",
    "source": "source",
    "legal": "flags.marked_for_legal",
    "sms": "flags.marked_for_sms",
    "language": "language",
}
SEARCH_CVS = [
    {'id': 'subject',
     'name': 'Subject',
     'field': 'subject',
     'list': 'subjectcodes'},
    {'id': 'companycodes',
     'name': 'Company Codes',
     'field': 'company_codes',
     'list': 'company_codes'}]


def get_index(repos=None):
    """Get index id for all repos."""
    if repos is None:
        repos = REPOS
    indexes = {app.data.elastic.index}
    for repo in repos:
        indexes.add(app.config['ELASTICSEARCH_INDEXES'].get(repo, app.data.elastic.index))
    return ','.join(indexes)


def get_docs(query_result):
    """Get documents from ES query result

    :param dict query_result: ES query result, as returned by service.search
    :return list: found documents
    """
    try:
        docs = [h['_source'] for h in query_result['hits']['hits']]
    except KeyError:
        logger.warning(u"Can't retrieve doc from ES results: {data}".format(data=query_result))
        docs = []
    return docs


def filter2query(filter_, user_id=None):
    """Convert filter syntax to ElasticSearch query

    :param dict filter_: filter syntax, as used in saved_searches endpoint
    :return dict: ElasticSearch query DSL usable with service.search method
    """
    # we'll delete key while we handle them, to check that all has been managed at the end
    search_query = filter_['query'].copy()
    query_must = []
    query_must_not = []
    post_filter = []
    post_filter_must_not = []

    # controlled vocabularies can be overriden in settings
    search_cvs = app.config.get('search_cvs', SEARCH_CVS)

    for cv in search_cvs:
        if cv['id'] in search_query and cv['field'] != cv['id']:
            terms = json.loads(search_query.pop(cv['id']))
            query_must.append({"terms": {cv['field'] + ".qcode": terms}})

    try:
        query_string = search_query.pop('q')
    except KeyError:
        pass
    else:
        for cv in search_cvs:
            if cv['field'] != cv['id']:
                query_string.replace(cv['id'] + '.qcode:(', cv['field'] + 'q.code:(')
        query_must.append(
            {"query_string": {
                "query": query_string,
                "default_operator": "AND"}})

    to_delete = []
    for key, value in search_query.items():
        if key == 'from_desk':
            desk = value.split('-')
            if len(desk) == 2:
                if desk[1] == 'authoring':
                    query_must.append({'term': {'task.last_authoring_desk': desk[0]}})
                else:
                    query_must.append({'term': {'task.last_production_desk': desk[0]}})
            else:
                logger.warning('unexpected "from_desk" value: {value}'.format(value=value))
        elif key == 'to_desk':
            desk = value.split('-')
            if len(desk) == 2:
                query_must.append({'term': {'task.desk': desk[0]}})
                if 'from_desk' not in filter_['query']:
                    if desk[1] == 'authoring':
                        field = 'task.last_production_desk'
                    else:
                        field = 'task.last_authoring_desk'
                    query_must.append({'exists': {field: field}})
            else:
                logger.warning('unexpected "from_desk" value: {value}'.format(value=value))
        elif key == 'spike':
            if value == 'include':
                pass
            elif value == 'only':
                query_must.append({"term": {"state": "spiked"}})
            else:
                query_must_not.append({"term": {"state": "spiked"}})
        elif key == 'featuremedia' and value:
            query_must.append({"exists": {"field": "associations.featuremedia"}})
        elif key == 'subject':
            terms = json.loads(value)
            query_must.append({"bool": {
                "should": [
                    {"terms": {"subject.qcode": terms}},
                    {"terms": {"subject.parent": terms}}],
                "minimum_should_match": 1}})
        elif key == 'company_codes':
            terms = json.loads(value)
            query_must.append({"terms": {"company_codes.qcode": terms}})
        elif key == 'marked_desks':
            terms = json.loads(value)
            query_must.append({"terms": {"marked_desks.desk_id": terms}})
        elif key == 'ignoreKilled':
            query_must_not.append({"terms": {"state": ["killed", "recalled"]}})
        elif key == 'onlyLastPublished':
            query_must_not.append({"term": {"last_published_version": "false"}})
        elif key == 'ignoreScheduled':
            query_must_not.append({"term": {"state": "scheduled"}})
        else:
            continue
        to_delete.append(key)

    for key in to_delete:
        del search_query[key]

    for key, field in POST_FILTER_MAP.items():
        value = search_query.pop(key, None)
        if value is not None:
            post_filter.append({'terms': {field: json.loads(value)}})
        else:
            value = search_query.pop('not' + key, None)
            if value is not None:
                post_filter_must_not.append({'terms': {field: json.loads(value)}})

    # used by AAP multimedia datalayer
    credit_qcode = search_query.pop('creditqcode', None)
    if credit_qcode is not None:
        values = json.loads(credit_qcode)
        post_filter.append({'terms': {'credit': [v['value'] for v in values]}})

    # remove other users drafts
    if user_id is not None:
        query_must.append({"bool": {
            "should": [
                {"bool": {
                    "must": [
                        {"term": {"state": "draft"}},
                        {"term": {"original_creator": user_id}}]}},
                {"bool": {
                    "must_not": {"terms": {"state": ["draft"]}}}}]}})

    # this is needed for archived collection
    query_must_not.append({"term": {"package_type": "takes"}})

    query = {
        "query": {
            "bool": {
                "must": query_must,
                "must_not": query_must_not
            }
        }
    }
    if post_filter or post_filter_must_not:
        query['post_filter'] = {"bool": {
            "must": post_filter,
            "must_not": post_filter_must_not}}

    if search_query:
        logger.warning("All query fields have not been used, remaining fields: {search_query}".format(
            search_query=search_query))

    return query



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import app_components
from .base_component import BaseComponent


def register_component(component):
    """Register a component - to be used in bootstrap.

    @param component: object
        The component object.
    """
    assert isinstance(component, BaseComponent), 'Invalid component %s' % component
    app_components[component.name()] = component


def get_component(name):
    """Return an instance of the component identified by given name."""
    if isinstance(name, BaseComponent) or (type(name) is type and issubclass(name, BaseComponent)):
        name = name.name()
    return app_components[name]



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
from superdesk.tests import TestCase
from superdesk.io.feed_parsers.ana_mpe_newsml import ANANewsMLOneFeedParser
from superdesk.etree import etree


class TestANACase(TestCase):
    def setUp(self):
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', 'ana1.xml'))
        provider = {'name': 'Test', 'source': 'ANA'}
        with open(fixture, 'rb') as f:
            self.item = ANANewsMLOneFeedParser().parse(etree.fromstring(f.read()), provider)

    def test_headline(self):
        self.assertEqual(self.item['headline'], 'Wage employment balance positive in Jan-March')
        self.assertEqual(self.item['subject'][0], {'qcode': '04008004', 'name': 'economic indicator'})
        self.assertEqual(self.item['dateline']['text'], 'ATHENS, April 4 ANA -')
        self.assertTrue(self.item['body_html'].startswith('<p>Wage employment'))
        self.assertTrue(self.item['body_html'].endswith(' job positions.</p>\n<p> </p>'), )
        self.assertEqual(self.item['word_count'], '115')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from apps.archive.archive import ArchiveResource, SOURCE as ARCHIVE
from superdesk.metadata.utils import item_url
import logging
from functools import partial
from flask import request, current_app as app
from superdesk import get_resource_service, Service, config
from superdesk.errors import SuperdeskApiError
from superdesk.metadata.item import CONTENT_TYPE, ITEM_TYPE, ITEM_STATE, CONTENT_STATE
from superdesk.publish import SUBSCRIBER_TYPES
from apps.archive.common import is_genre, BROADCAST_GENRE, ITEM_RESEND
from apps.publish.enqueue import get_enqueue_service
from apps.archive.common import ITEM_OPERATION

logger = logging.getLogger(__name__)


class ResendResource(ArchiveResource):
    endpoint_name = 'archive_resend'
    resource_title = endpoint_name

    schema = {
        'subscribers': {'type': 'list'},
        'version': {'type': 'integer'}
    }

    url = 'archive/<{0}:original_id>/resend'.format(item_url)
    resource_methods = ['POST']
    privileges = {'POST': 'resend'}


class ResendService(Service):

    digital = partial(filter, lambda s: (s.get('subscriber_type', '') in {SUBSCRIBER_TYPES.DIGITAL,
                                                                          SUBSCRIBER_TYPES.ALL}))

    def create(self, docs, **kwargs):
        doc = docs[0] if len(docs) > 0 else {}
        article_id = request.view_args['original_id']
        article_version = doc.get('version')
        article = self._validate_article(article_id, article_version)
        subscribers = self._validate_subscribers(doc.get('subscribers'), article)
        get_enqueue_service(article.get(ITEM_OPERATION)).resend(article, subscribers)
        app.on_archive_item_updated({'subscribers': doc.get('subscribers')}, article, ITEM_RESEND)
        return [article_id]

    def _validate_subscribers(self, subscriber_ids, article):
        if not subscriber_ids:
            raise SuperdeskApiError.badRequestError(message='No subscribers selected!')

        query = {'$and': [{config.ID_FIELD: {'$in': list(subscriber_ids)}}, {'is_active': True}]}
        subscribers = list(get_resource_service('subscribers').get(req=None, lookup=query))

        if len(subscribers) == 0:
            raise SuperdeskApiError.badRequestError(message='No active subscribers found!')

        if is_genre(article, BROADCAST_GENRE):
            digital_subscribers = list(self.digital(subscribers))
            if len(digital_subscribers) > 0:
                raise SuperdeskApiError.badRequestError('Only wire subscribers can receive broadcast stories!')

        return subscribers

    def _validate_article(self, article_id, article_version):
        archive_article = get_resource_service(ARCHIVE).find_one(req=None, _id=article_id)

        if not archive_article:
            raise SuperdeskApiError.badRequestError(message="Story couldn't be found!")

        if archive_article[ITEM_TYPE] not in [CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED]:
            raise SuperdeskApiError.badRequestError(
                message='Only text stories can be resent!')

        if archive_article.get(ITEM_STATE) not in \
                [CONTENT_STATE.PUBLISHED, CONTENT_STATE.CORRECTED, CONTENT_STATE.KILLED]:
            raise SuperdeskApiError.badRequestError(
                message='Only published, corrected or killed stories can be resent!')

        if archive_article[config.VERSION] != article_version:
            raise SuperdeskApiError.badRequestError(
                message='Please use the newest version {} to resend!'.format(archive_article[config.VERSION]))

        if archive_article.get('rewritten_by'):
            raise SuperdeskApiError.badRequestError(
                message='Updated story cannot be resent!')

        return archive_article



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import mimetypes
from datetime import datetime
from superdesk.tests import TestCase
from superdesk import get_resource_service
from superdesk.media.media_operations import process_file_from_stream
import os

from superdesk.metadata.item import GUID_TAG, ITEM_TYPE, CONTENT_TYPE
from superdesk.media.renditions import generate_renditions, get_renditions_spec
from superdesk.metadata import utils
from superdesk.upload import url_for_media
from superdesk import filemeta
from flask import current_app as app
import json
import hashlib


class BaseMediaEditorTestCase(TestCase):

    def setUp(self):
        super().setUp()
        dirname = os.path.dirname(os.path.realpath(__file__))
        image_path = os.path.normpath(os.path.join(dirname, 'fixtures', self.filename))
        content_type = mimetypes.guess_type(image_path)[0]
        guid = utils.generate_guid(type=GUID_TAG)
        self.item = {'guid': guid,
                     'version': 1,
                     '_id': guid,
                     ITEM_TYPE: CONTENT_TYPE.PICTURE,
                     'mimetype': content_type,
                     'versioncreated': datetime.now()
                     }

        with open(image_path, 'rb') as f:
            _, content_type, file_metadata = process_file_from_stream(f, content_type=content_type)
            f.seek(0)
            file_id = app.media.put(f, filename=self.filename, content_type=content_type, metadata=file_metadata)
            filemeta.set_filemeta(self.item, file_metadata)
            f.seek(0)
            rendition_spec = get_renditions_spec()
            renditions = generate_renditions(f, file_id, [file_id], 'image',
                                             content_type, rendition_spec, url_for_media)
            self.item['renditions'] = renditions
        archive = get_resource_service('archive')
        archive.post([self.item])

    def do_edit(self, edit, item=None):
        """Helper method to test edition on current test media

        :param dict edit: edition instructions
        :return dict item: item with modified media
        """
        media_editor = get_resource_service('media_editor')
        request_data = {
            'edit': edit}
        if item is None:
            item_id = self.item['_id']
            request_data['item_id'] = item_id
        else:
            for k in item:
                if isinstance(item[k], datetime):
                    item[k] = item[k].isoformat()
            for r in item['renditions'].values():
                r['media'] = str(r['media'])
            request_data['item'] = item
        docs = [request_data]
        with self.app.test_request_context('media_editor',
                                           method='POST',
                                           content_type='application/json',
                                           data=json.dumps(request_data)):
            media_editor.create(docs)

        return docs[0]

    def md5_sum(self, item, rendition):
        """Generate MD5 sum

        useful to check that image has been transformed correctly
        :param str rendition: name of the rendition to use
        """
        media_id = item['renditions'][rendition]['media']
        media = app.media.get(media_id)
        md5_hash = hashlib.md5(media.read())
        return md5_hash.hexdigest()


class MediaEditorTestCase(BaseMediaEditorTestCase):
    filename = "IPTC-PhotometadataRef-Std2017.1.jpg"

    def test_edition(self):
        """Test basic edition instructions"""
        item = self.do_edit({"contrast": 1.2, "rotate": "90"})
        md5_hash = self.md5_sum(item, 'original')
        self.assertEqual(md5_hash, 'e3bae72827b39918f02211936645822b')

    def test_saturation(self):
        """Test saturation change"""
        item = self.do_edit({"saturation": 0})
        md5_hash = self.md5_sum(item, 'original')
        self.assertEqual(md5_hash, '9693480b4e15843bed520fa1a159fb5a')

    def test_update(self):
        """Test that item is updated correctly"""
        original_media_id = self.item['renditions']['original']['media']
        item = self.do_edit({"rotate": "170", "saturation": "0"}, item=self.item)
        expected_media_id = item['renditions']['original']['media']
        self.assertNotEqual(original_media_id, expected_media_id)
        md5_hash = self.md5_sum(item, 'original')
        self.assertEqual(md5_hash, '4e88727bb595d21ecdc7670a6afb099c')
        archive_service = get_resource_service('archive')
        updated_item = next(archive_service.find({'_id': item['_id']}))
        self.assertEqual(updated_item['renditions']['original']['media'], expected_media_id)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""
Module which contains metadata schema for core superdesk entities.
"""



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
import unittest

from xml.etree import ElementTree
from superdesk.io.feed_parsers.newsml_2_0 import NewsMLTwoFeedParser


class BaseNewMLTwoTestCase(unittest.TestCase):
    def setUp(self):
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', self.filename))
        provider = {'name': 'Test'}
        with open(fixture, 'rb') as f:
            self.parser = NewsMLTwoFeedParser()
            self.xml = ElementTree.parse(f)
            self.item = self.parser.parse(self.xml.getroot(), provider)


class ReutersTestCase(BaseNewMLTwoTestCase):
    filename = 'tag:reuters.com,0000:newsml_L4N1FL0N0:1132689232'

    def test_content(self):
        self.assertEqual(self.item[0].get('headline'), "PRECIOUS-Gold rises as Trump policy fuels safe haven demand")
        self.assertEqual(self.item[0].get('guid'), 'tag:reuters.com,0000:newsml_L4N1FL0N0:1132689232')
        self.assertEqual(self.item[0].get('uri'), 'tag:reuters.com,0000:newsml_L4N1FL0N0')
        self.assertEqual(self.item[0].get('firstcreated').isoformat(), '2017-01-31T03:47:29')
        self.assertTrue(self.item[0].get('body_html').startswith('<p>(Adds comments, detail)</p>'))

    def test_can_parse(self):
        self.assertTrue(NewsMLTwoFeedParser().can_parse(self.xml.getroot()))


class ResutersResultsTestCase(BaseNewMLTwoTestCase):
    filename = 'tag:reuters.com,0000:newsml_ISS149709:1618095828'

    def test_results(self):
        self.assertTrue(self.item[0].get('body_html').startswith(
            '<p>Jan 30 (Gracenote) - Results and standings from the Turkish championship matches on Monday <br/>'))


class ANSATestCase(BaseNewMLTwoTestCase):
    filename = 'ansa-newsmlg2-text.xml'

    def test_language(self):
        self.assertEqual('it', self.item[0]['language'])


class ReutersOptaTestCase(BaseNewMLTwoTestCase):
    filename = 'tag:reuters.com,2018:newsml_MTZXEE13ZXCZES:2'

    def test_body(self):
        self.assertTrue(self.item[0].get('body_html').startswith('<pre>Jan 3 (OPTA) - Results and fixtures for the '
                                                                 'Primeira'))


class IPTCExampleTextTestCase(BaseNewMLTwoTestCase):
    filename = 'LISTING 1 A NewsML-G2 News Item.xml'

    def test_news_item_parsing(self):
        self.assertEqual(1, len(self.item))
        item = self.item[0]
        self.assertEqual('urn:newsml:acmenews.com:20161018:US-FINANCE-FED', item['uri'])
        self.assertEqual('2016-10-21T16:25:32-05:00', item['versioncreated'].isoformat())
        self.assertIn('STRICTLY EMBARGOED', item['ednote'])
        self.assertEqual(1, len(item['authors']))

    def test_can_parse(self):
        self.assertTrue(self.parser.can_parse(self.xml.getroot()))


class IPTCExamplePackage(BaseNewMLTwoTestCase):
    filename = 'LISTING 6 Simple NewsML-G2 Package.xml'

    def test_news_item_parsing(self):
        self.assertEqual(1, len(self.item))

    def test_can_parse(self):
        self.assertTrue(self.parser.can_parse(self.xml.getroot()))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import datetime
import traceback
import superdesk
import requests
from flask import current_app as app

from superdesk.errors import IngestApiError
from superdesk.etree import etree, ParseError
from superdesk.io.registry import register_feeding_service, register_feeding_service_parser
from superdesk.io.feeding_services.http_service import HTTPFeedingService
from superdesk.logging import logger
from superdesk.utc import utcnow
from urllib.parse import urlparse, urlunparse


requests.packages.urllib3.disable_warnings()


class ReutersHTTPFeedingService(HTTPFeedingService):
    """
    Feeding Service class which can read article(s) using HTTP provided by Reuters.
    """

    NAME = 'reuters_http'

    ERRORS = [IngestApiError.apiTimeoutError().get_error_description(),
              IngestApiError.apiRedirectError().get_error_description(),
              IngestApiError.apiRequestError().get_error_description(),
              IngestApiError.apiUnicodeError().get_error_description(),
              IngestApiError.apiParseError().get_error_description(),
              IngestApiError.apiGeneralError().get_error_description()]

    DATE_FORMAT = '%Y.%m.%d.%H.%M'

    label = 'Reuters feed API'

    fields = [
        {
            'id': 'url', 'type': 'text', 'label': 'Feed URL',
            'placeholder': 'Feed URL', 'required': True,
            'default': 'http://rmb.reuters.com/rmd/rest/xml'
        },
        {
            'id': 'auth_url', 'type': 'text', 'label': 'URL for Authentication',
            'placeholder': 'authentication url', 'required': True,
            'default': 'https://commerce.reuters.com/rmd/rest/xml/login'
        },
        {
            'id': 'username', 'type': 'text', 'label': 'Username',
            'placeholder': 'Username', 'required': True
        },
        {
            'id': 'password', 'type': 'password', 'label': 'Password',
            'placeholder': 'Password', 'required': True
        }
    ]

    session = None

    def _update(self, provider, update):
        updated = utcnow()

        last_updated = provider.get('last_updated')
        ttl_minutes = app.config['INGEST_EXPIRY_MINUTES']
        if not last_updated or last_updated < updated - datetime.timedelta(minutes=ttl_minutes):
            last_updated = updated - datetime.timedelta(minutes=ttl_minutes)

        self.provider = provider
        provider_config = provider.get('config')
        if not provider_config:
            provider_config = {}
            provider['config'] = provider_config

        provider_config.setdefault('url', 'http://rmb.reuters.com/rmd/rest/xml')
        provider_config.setdefault('auth_url', 'https://commerce.reuters.com/rmd/rest/xml/login')
        self.URL = provider_config.get('url')

        for channel in self._get_channels():
            ids = self._get_article_ids(channel, last_updated, updated)
            for id in ids:
                try:
                    items = self.fetch_ingest(id)
                    if items:
                        yield items
                # if there was an exception processing the one of the bunch log it and continue
                except Exception as ex:
                    logger.warn('Reuters item {} has not been retrieved'.format(id))
                    logger.exception(ex)

    def _get_channels(self):
        """Get subscribed channels."""
        channels = []
        tree = self._get_tree('channels')
        for channel in tree.findall('channelInformation'):
            channels.append(channel.find('alias').text)

        return channels

    def _get_tree(self, endpoint, payload=None):
        """Get xml response for given API endpoint and payload.

        :param: endpoint
        :type endpoint: str
        :param: payload
        :type payload: str
        """

        if payload is None:
            payload = {}

        payload['token'] = self._get_auth_token(self.provider, update=True)
        url = self._get_absolute_url(endpoint)

        if not self.session:
            self.session = requests.Session()

        retries = 0
        while True:
            try:
                response = self.session.get(url, params=payload, timeout=(30, 15))
            except requests.exceptions.Timeout as ex:
                if retries < 3:
                    logger.warn('Reuters API timeout retrying, retries {}'.format(retries))
                    retries += 1
                    continue
                raise IngestApiError.apiTimeoutError(ex, self.provider)
            except requests.exceptions.TooManyRedirects as ex:
                # Tell the user their URL was bad and try a different one
                raise IngestApiError.apiRedirectError(ex, self.provider)
            except requests.exceptions.RequestException as ex:
                # catastrophic error. bail.
                raise IngestApiError.apiRequestError(ex, self.provider)
            except Exception as error:
                traceback.print_exc()
                raise IngestApiError.apiGeneralError(error, self.provider)

            if response.status_code == 404:
                raise LookupError('Not found %s' % payload)

            break

        try:
            return etree.fromstring(response.content)  # workaround for http mock lib
        except UnicodeEncodeError as error:
            traceback.print_exc()
            raise IngestApiError.apiUnicodeError(error, self.provider)
        except ParseError as error:
            traceback.print_exc()
            raise IngestApiError.apiParseError(error, self.provider)
        except Exception as error:
            traceback.print_exc()
            raise IngestApiError.apiGeneralError(error, self.provider)

    def _get_absolute_url(self, endpoint):
        """
        Get absolute URL for given endpoint.

        :param: endpoint
        :type endpoint: str
        """
        return '/'.join([self.URL, endpoint])

    def _get_article_ids(self, channel, last_updated, updated):
        """
        Get article ids which should be upserted also save the poll token that is returned.
        """
        ids = set()
        payload = {'channel': channel, 'fieldsRef': 'id'}

        # check if the channel has a pollToken if not fall back to dateRange
        last_poll_token = self._get_poll_token(channel)
        if last_poll_token is not None:
            logger.info("Reuters requesting channel {} with poll token {}".format(channel, last_poll_token))
            payload['pollToken'] = last_poll_token
        else:
            payload['dateRange'] = "%s-%s" % (self._format_date(last_updated), self._format_date(updated))
            logger.info("Reuters requesting channel {} with dateRange {}".format(channel, payload['dateRange']))

        tree = self._get_tree('items', payload)
        status_code = tree.find('status').get('code') if tree.tag == 'results' else tree.get('code')
        # check the returned status
        if status_code != '10':
            logger.warn("Reuters channel request returned status code {}".format(status_code))
            # status code 30 indicates failure
            if status_code == '30':
                # invalid token
                logger.warn("Reuters error on channel {} code {} {}".format(channel, tree.find('error').get('code'),
                                                                            tree.find('error').text))
                if tree.find('error').get('code') == '2100':
                    self._save_poll_token(channel, None)
                    logger.warn("Reuters channel invalid token reseting {}".format(status_code))
                return ids

        # extract the returned poll token if there is one
        poll_token = tree.find('pollToken')
        if poll_token is not None:
            # a new token indicated new content
            if poll_token.text != last_poll_token:
                logger.info("Reuters channel {} new token {}".format(channel, poll_token.text))
                self._save_poll_token(channel, poll_token.text)
            else:
                # the token has not changed, so nothing new
                logger.info("Reuters channel {} nothing new".format(channel))
                return ids
        else:
            logger.info("Reuters channel {} retrieved no token".format(channel))
            return ids

        for result in tree.findall('result'):
            id = result.find('id').text
            ids.add(id)
            logger.info("Reuters id : {}".format(id))

        return ids

    def _save_poll_token(self, channel, poll_token):
        """Saves the poll token for the passed channel in the config section of the

        :param channel:
        :param poll_token:
        :return:
        """
        # get the provider in case it has been updated by another channel
        ingest_provider_service = superdesk.get_resource_service('ingest_providers')
        provider = ingest_provider_service.find_one(req=None, _id=self.provider[superdesk.config.ID_FIELD])
        provider_token = provider.get('tokens')
        if 'poll_tokens' not in provider_token:
            provider_token['poll_tokens'] = {channel: poll_token}
        else:
            provider_token['poll_tokens'][channel] = poll_token
        upd_provider = {'tokens': provider_token}
        ingest_provider_service.system_update(self.provider[superdesk.config.ID_FIELD], upd_provider, self.provider)

    def _get_poll_token(self, channel):
        """Get the poll token from provider config if it is available.

        :param channel:
        :return: token
        """
        if 'tokens' in self.provider and 'poll_tokens' in self.provider['tokens']:
            return self.provider.get('tokens').get('poll_tokens').get(channel, None)

    def _format_date(self, date):
        return date.strftime(self.DATE_FORMAT)

    def fetch_ingest(self, id):
        items = self._parse_items(id)
        result_items = []
        while items:
            item = items.pop()
            self.add_timestamps(item)
            try:
                items.extend(self._fetch_items_in_package(item))
                result_items.append(item)
            except LookupError as err:
                self.log_item_error(err, item, self.provider)
                return []

        return result_items

    def _parse_items(self, id):
        """
        Parse item message and return given items.
        """

        payload = {'id': id}
        tree = self._get_tree('item', payload)

        parser = self.get_feed_parser(self.provider, tree)
        items = parser.parse(tree, self.provider)

        return items

    def _fetch_items_in_package(self, item):
        """
        Fetch remote assets for given item.
        """
        items = []
        for group in item.get('groups', []):
            for ref in group.get('refs', []):
                if 'residRef' in ref:
                    items.extend(self._parse_items(ref.get('residRef')))

        return items

    def prepare_href(self, href, mimetype=None):
        (scheme, netloc, path, params, query, fragment) = urlparse(href)
        new_href = urlunparse((scheme, netloc, path, '', '', ''))
        return '%s?auth_token=%s' % (new_href, self._get_auth_token(self.provider, update=True))


register_feeding_service(ReutersHTTPFeedingService)
register_feeding_service_parser(ReutersHTTPFeedingService.NAME, 'newsml2')




import superdesk

from datetime import timedelta
from superdesk.utc import utcnow
from superdesk.utils import get_random_token
from content_api import MONGO_PREFIX


class SubscriberTokenResource(superdesk.Resource):
    schema = {
        '_id': {'type': 'string', 'unique': True},
        'expiry': {'type': 'datetime'},
        'subscriber': superdesk.Resource.rel('subscribers', required=True),
    }

    item_url = 'regex(".+")'
    resource_methods = ['GET', 'POST']
    item_methods = ['GET', 'DELETE']
    privileges = {'POST': 'subscribers', 'DELETE': 'subscribers'}

    datasource = {
        'default_sort': [('_created', 1)],
    }

    mongo_prefix = MONGO_PREFIX


class SubscriberTokenService(superdesk.Service):

    def create(self, docs, **kwargs):
        for doc in docs:
            doc['_id'] = get_random_token()
            doc.setdefault('expiry', utcnow() + timedelta(days=7))
        return super().create(docs, **kwargs)



from flask import Blueprint, request, Response
import superdesk
from superdesk import get_resource_service
from superdesk.publish.formatters import get_formatter
from apps.content_types import apply_schema


bp = Blueprint('format_document', __name__)


def get_mime_type(formatter_qcode):
    if(formatter_qcode == 'newsmlg2'):
        return 'text/xml'
    else:
        return 'application/json'


@bp.route('/format-document-for-preview/', methods=['GET', 'OPTIONS'])
def format_document():

    document_id = request.args.get('document_id')
    subscriber_id = request.args.get('subscriber_id')
    formatter_qcode = request.args.get('formatter')

    subscriber = get_resource_service('subscribers').find_one(req=None, _id=subscriber_id)
    doc = get_resource_service('archive').find_one(req=None, _id=document_id)

    formatter = get_formatter(formatter_qcode, doc)
    formatted_docs = formatter.format(article=apply_schema(doc),
                                      subscriber=subscriber,
                                      codes=None)

    headers = {
        'Access-Control-Allow-Origin': '*',
        'Access-Control-Allow-Methods': 'GET',
        'Cache-Control': 'no-cache, no-store, must-revalidate'
    }

    return Response(formatted_docs[0][1], headers=headers, mimetype=get_mime_type(formatter_qcode))


def init_app(app):

    superdesk.blueprint(bp, app)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import logging
import superdesk
from superdesk.celery_app import celery
from superdesk.lock import lock, unlock
from .saved_searches import SavedSearchesService, SavedSearchesResource, AllSavedSearchesResource, \
    SavedSearchItemsResource, SavedSearchItemsService, AllSavedSearchesService
from superdesk import get_resource_service
from superdesk import es_utils
import pytz
from croniter import croniter
from datetime import datetime
from flask import render_template
from superdesk import emails
import json

logger = logging.getLogger(__name__)
REPORT_SOFT_LIMIT = 60 * 5
LOCK_NAME = "saved_searches_report"


def init_app(app):
    endpoint_name = 'saved_searches'
    service = SavedSearchesService(endpoint_name, backend=superdesk.get_backend())
    SavedSearchesResource(endpoint_name, app=app, service=service)

    endpoint_name = 'all_saved_searches'
    service = AllSavedSearchesService(endpoint_name, backend=superdesk.get_backend())
    AllSavedSearchesResource(endpoint_name, app=app, service=service)

    endpoint_name = 'saved_search_items'
    service = SavedSearchItemsService(endpoint_name, backend=superdesk.get_backend())
    SavedSearchItemsResource(endpoint_name, app=app, service=service)

    superdesk.privilege(name='global_saved_searches',
                        label='Manage Global Saved Searches',
                        description='User can manage other users\' global saved searches')
    superdesk.privilege(name='saved_searches',
                        label='Manage Saved Searches',
                        description='User can manage saved searches')
    superdesk.privilege(name='saved_searches_subscriptions',
                        label='Manage Saved Searches Subscriptions',
                        description='User can (un)subscribe to saved searches')
    superdesk.privilege(name='saved_searches_subscriptions_admin',
                        label='Manage Saved Searches Subscriptions For Other Users',
                        description='User manage other users saved searches subscriptions ')


def get_next_date(scheduling, base=None):
    """Get next schedule date

    :param str scheduling: task schedule, using cron syntax
    :return datetime: date of next schedule
    """
    if base is None:
        tz = pytz.timezone(superdesk.app.config['DEFAULT_TIMEZONE'])
        base = datetime.now(tz=tz)
    cron_iter = croniter(scheduling, base)
    return cron_iter.get_next(datetime)


def send_report_email(user_id, search, docs):
    """Send saved search report by email.

    :param dict search: saved search data
    :param list found_items: items matching the search request
    """
    users_service = get_resource_service('users')
    user_data = next(users_service.find({'_id': user_id}))
    recipients = [user_data['email']]
    app = superdesk.app
    admins = app.config['ADMINS']
    app_name = app.config['APPLICATION_NAME']
    subject = "Saved searches report"
    text_body = render_template("saved_searches_report.txt", app_name=app_name, search=search, docs=docs)
    html_body = render_template("saved_searches_report.html", app_name=app_name, search=search, docs=docs)
    emails.send_email.delay(
        subject=subject, sender=admins[0], recipients=recipients, text_body=text_body, html_body=html_body)


def publish_report(user_id, search_data):
    """Create report for a search and send it by email"""
    query = es_utils.filter2query(json.loads(search_data['filter']), user_id=user_id)
    found = superdesk.app.data.elastic.es.search(
        body=query, index=es_utils.get_index())
    docs = es_utils.get_docs(found)
    send_report_email(user_id, search_data, docs)


@celery.task(soft_time_limit=REPORT_SOFT_LIMIT)
def report():
    """Check all saved_searches with subscribers, and publish reports"""
    if not lock(LOCK_NAME, expire=REPORT_SOFT_LIMIT + 10):
        return
    try:
        saved_searches = get_resource_service('saved_searches')
        subscribed_searches = saved_searches.find({"subscribers": {"$exists": 1}})
        tz = pytz.timezone(superdesk.app.config['DEFAULT_TIMEZONE'])
        now = datetime.now(tz=tz)
        for search in subscribed_searches:
            subscribed_users = search['subscribers'].get('user_subscriptions', [])
            do_update = False
            for user_data in subscribed_users:
                scheduling = user_data['scheduling']
                next_report = user_data.get('next_report')
                if next_report is None:
                    user_data['next_report'] = get_next_date(scheduling)
                    do_update = True
                elif next_report <= now:
                    publish_report(user_data['user'], search)
                    user_data['last_report'] = now
                    user_data['next_report'] = get_next_date(scheduling)
                    do_update = True
            if do_update:
                updates = {'subscribers': search['subscribers']}
                saved_searches.update(search['_id'], updates, search)
    except Exception as e:
        logger.error("Can't report saved searched: {reason}".format(reason=e))
    finally:
        unlock(LOCK_NAME)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from lxml import etree
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, FORMATS, FORMAT
from superdesk.etree import parse_html
from superdesk.text_utils import get_text
from superdesk.publish import registered_transmitters

formatters = []
logger = logging.getLogger(__name__)


class FormatterRegistry(type):
    """Registry metaclass for formatters."""

    def __init__(cls, name, bases, attrs):
        """Register sub-classes of Formatter class when defined."""
        super(FormatterRegistry, cls).__init__(name, bases, attrs)
        if name != 'Formatter':
            formatters.append(cls())


class Formatter(metaclass=FormatterRegistry):
    """Base Formatter class for all types of Formatters like News ML 1.2, News ML G2, NITF, etc."""

    def __init__(self):
        self.can_preview = False
        self.can_export = False
        self.destination = None
        self.subscriber = None

    def format(self, article, subscriber, codes=None):
        """Formats the article and returns the transformed string"""
        raise NotImplementedError()

    def export(self, article, subscriber, codes=None):
        """Formats the article and returns the output string for export"""
        raise NotImplementedError()

    def can_format(self, format_type, article):
        """Test if formatter can format for given article."""
        raise NotImplementedError()

    def append_body_footer(self, article):
        """
        Checks if the article has any Public Service Announcements and if available appends each of them to the body.

        :return: body with public service announcements.
        """
        try:
            article['body_html'] = article['body_html'].replace('<br>', '<br/>')
        except KeyError:
            pass

        body = ''
        if article[ITEM_TYPE] in [CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED]:
            body = article.get('body_html', '')
        elif article[ITEM_TYPE] in [CONTENT_TYPE.AUDIO, CONTENT_TYPE.PICTURE, CONTENT_TYPE.VIDEO]:
            body = article.get('description', '')

        if body and article.get(FORMAT, '') == FORMATS.PRESERVED:
            body = body.replace('\n', '\r\n').replace('\r\r', '\r')
            parsed = parse_html(body, content='html')

            for br in parsed.xpath('//br'):
                br.tail = '\r\n' + br.tail if br.tail else '\r\n'

            etree.strip_elements(parsed, 'br', with_tail=False)
            body = etree.tostring(parsed, encoding="unicode")

        if body and article.get('body_footer'):
            footer = article.get('body_footer')
            if article.get(FORMAT, '') == FORMATS.PRESERVED:
                body = '{}\r\n{}'.format(body, get_text(footer))
            else:
                body = '{}{}'.format(body, footer)
        return body

    def append_legal(self, article, truncate=False):
        """
        Checks if the article has the legal flag on and adds 'Legal:' to the slugline

        :param article: article having the slugline
        :param truncate: truncates the slugline to 24 characters
        :return: updated slugline
        """
        slugline = article.get('slugline', '') or ''

        if article.get('flags', {}).get('marked_for_legal', False):
            slugline = '{}: {}'.format('Legal', slugline)
            if truncate:
                slugline = slugline[:24]

        return slugline

    def map_html_to_xml(self, element, html):
        """
        Map the html text tags to xml

        :param etree.Element element: The xml element to populate
        :param str html: the html to parse the text from
        :return:
        """
        root = parse_html(html)
        # if there are no ptags just br
        if not len(root.xpath('//p')) and len(root.xpath('//br')):
            para = etree.SubElement(element, 'p')
            for br in root.xpath('//br'):
                etree.SubElement(para, 'br').text = br.text

        for p in root.xpath('//p'):
            para = etree.SubElement(element, 'p')
            if len(p.xpath('.//br')) > 0:
                for br in p.xpath('.//br'):
                    etree.SubElement(para, 'br').text = br.text
            para.text = etree.tostring(p, encoding="unicode", method="text")

        # there neither ptags pr br's
        if len(list(element)) == 0:
            etree.SubElement(element, 'p').text = etree.tostring(root, encoding="unicode", method="text")

    def set_destination(self, destination=None, subscriber=None):
        self.destination = destination
        self.subscriber = subscriber

    def _publish_media(self, media):
        if self.destination:
            try:
                transmitter = registered_transmitters[self.destination['delivery_type']]
            except KeyError:
                logger.warning('Missing transmitter for destination %s', self.destination)
            else:
                return transmitter.transmit_media(media, self.subscriber, self.destination)


def get_formatter(format_type, article):
    """Get parser for given xml.

    :param etree: parsed xml
    """
    for formatter in formatters:
        if formatter.can_format(format_type, article):
            return formatter


def get_all_formatters():
    """Return all formatters registered."""
    return formatters


from .nitf_formatter import NITFFormatter  # NOQA
from .ninjs_formatter import NINJSFormatter  # NOQA
from .newsml_1_2_formatter import NewsML12Formatter  # NOQA
from .newsml_g2_formatter import NewsMLG2Formatter  # NOQA
from .email_formatter import EmailFormatter  # NOQA
from .ninjs_newsroom_formatter import NewsroomNinjsFormatter  # NOQA
from .idml_formatter import IDMLFormatter  # NOQA



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""Superdesk default settings, those can be overriden via settings file in project or env variable.

Environment variables names match config name, with some expections documented below.
"""
import json
import os
import pytz
import tzlocal

from datetime import timedelta, datetime
from celery.schedules import crontab
from kombu import Queue, Exchange
from distutils.util import strtobool as _strtobool

try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse


def strtobool(value):
    try:
        return bool(_strtobool(value))
    except ValueError:
        return False


def env(variable, fallback_value=None):
    if os.environ.get('SUPERDESK_USE_DEFAULTS'):
        return fallback_value

    env_value = os.environ.get(variable)
    if env_value is None:
        return fallback_value
    # Next isn't needed anymore
    elif env_value == "__EMPTY__":
        return ''
    else:
        return env_value


def celery_queue(name):
    """Get celery queue name with optional prefix set in environment.

    If you want to use multiple workers in Procfile you have to use the prefix::

        work_publish: celery -A worker -Q "${SUPERDESK_CELERY_PREFIX}publish" worker
        work_default: celery -A worker worker

    :param name: queue name
    """
    return "{}{}".format(os.environ.get('SUPERDESK_CELERY_PREFIX', ''), name)


#: Default TimeZone, will try to guess from server settings if not set
DEFAULT_TIMEZONE = env('DEFAULT_TIMEZONE')

if DEFAULT_TIMEZONE is None:
    DEFAULT_TIMEZONE = tzlocal.get_localzone().zone

if not DEFAULT_TIMEZONE:
    raise ValueError("DEFAULT_TIMEZONE is empty")


def local_to_utc_hour(hour):
    now = datetime.now()
    tz = pytz.timezone(DEFAULT_TIMEZONE)
    local = tz.localize(now.replace(hour=hour))
    return local.utctimetuple()[3]


ABS_PATH = os.path.abspath(os.path.join(os.path.dirname(__file__), '../'))
BEHAVE_TESTS_FIXTURES_PATH = ABS_PATH + '/features/steps/fixtures'

XML = False
IF_MATCH = True
BANDWIDTH_SAVER = False
DATE_FORMAT = '%Y-%m-%dT%H:%M:%S+0000'
ELASTIC_DATE_FORMAT = '%Y-%m-%d'
ELASTIC_DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%S'
PAGINATION_LIMIT = 200

LOG_CONFIG_FILE = env('LOG_CONFIG_FILE', 'logging_config.yml')
LOG_SERVER_ADDRESS = env('LOG_SERVER_ADDRESS', 'localhost')
LOG_SERVER_PORT = int(env('LOG_SERVER_PORT', 5555))

#: application name - used in email footers, ``APP_NAME`` env
APPLICATION_NAME = env('APP_NAME', 'Superdesk')
#: public client url - used to create links within emails etc, ``SUPERDESK_CLIENT_URL`` env
CLIENT_URL = env('SUPERDESK_CLIENT_URL', 'http://localhost:9000')

SERVER_URL = env('SUPERDESK_URL', 'http://localhost:5000/api')
server_url = urlparse(SERVER_URL)
SERVER_DOMAIN = server_url.netloc or 'localhost'
URL_PREFIX = env('URL_PREFIX', server_url.path.lstrip('/')) or ''

VALIDATION_ERROR_STATUS = 400
JSON_SORT_KEYS = False

CACHE_CONTROL = 'max-age=0, no-cache'

X_DOMAINS = '*'
X_MAX_AGE = 24 * 3600
X_HEADERS = ['Content-Type', 'Authorization', 'If-Match']

#: mongo db name, only used when mongo_uri is not set
MONGO_DBNAME = env('MONGO_DBNAME', 'superdesk')

#: full mongodb connection uri, overrides ``MONGO_DBNAME`` if set
MONGO_URI = env('MONGO_URI', 'mongodb://localhost/%s' % MONGO_DBNAME)

#: allow all mongo queries
MONGO_QUERY_BLACKLIST = []

#: legal archive switch
LEGAL_ARCHIVE = env('LEGAL_ARCHIVE', None)

#: legal archive db name
LEGAL_ARCHIVE_DBNAME = env('LEGAL_ARCHIVE_DBNAME', 'legal_archive')

#: legal archive mongodb uri
LEGAL_ARCHIVE_URI = env('LEGAL_ARCHIVE_URI', 'mongodb://localhost/%s' % LEGAL_ARCHIVE_DBNAME)

#: archived mongodb db name
ARCHIVED_DBNAME = env('ARCHIVED_DBNAME', 'archived')

#: archived mongodb uri
ARCHIVED_URI = env('ARCHIVED_URI', 'mongodb://localhost/%s' % ARCHIVED_DBNAME)

CONTENTAPI_MONGO_DBNAME = 'contentapi'
CONTENTAPI_MONGO_URI = env('CONTENTAPI_MONGO_URI', 'mongodb://localhost/%s' % CONTENTAPI_MONGO_DBNAME)

#: elastic url
ELASTICSEARCH_URL = env('ELASTICSEARCH_URL', 'http://localhost:9200')
CONTENTAPI_ELASTICSEARCH_URL = env('CONTENTAPI_ELASTICSEARCH_URL', ELASTICSEARCH_URL)

#: elastic index name
ELASTICSEARCH_INDEX = env('ELASTICSEARCH_INDEX', 'superdesk')
CONTENTAPI_ELASTICSEARCH_INDEX = env('CONTENTAPI_ELASTICSEARCH_INDEX', CONTENTAPI_MONGO_DBNAME)

if env('ELASTIC_PORT'):
    ELASTICSEARCH_URL = env('ELASTIC_PORT').replace('tcp:', 'http:')

ELASTICSEARCH_BACKUPS_PATH = env('ELASTICSEARCH_BACKUPS_PATH', '')

#: elastic settings - superdesk custom filter
ELASTICSEARCH_SETTINGS = {
    'settings': {
        'analysis': {
            'filter': {
                'remove_hyphen': {
                    'pattern': '[-]',
                    'type': 'pattern_replace',
                    'replacement': ' '
                }
            },
            'analyzer': {
                'phrase_prefix_analyzer': {
                    'type': 'custom',
                    'filter': ['remove_hyphen', 'lowercase'],
                    'tokenizer': 'keyword'
                }
            }
        }
    }
}

#: redis url
REDIS_URL = env('REDIS_URL', 'redis://localhost:6379')
if env('REDIS_PORT'):
    REDIS_URL = env('REDIS_PORT').replace('tcp:', 'redis:')

#: cache url - superdesk will try to figure out if it's redis or memcached
CACHE_URL = env('SUPERDESK_CACHE_URL', REDIS_URL)

#: celery broker
BROKER_URL = env('CELERY_BROKER_URL', REDIS_URL)
CELERY_BROKER_URL = BROKER_URL

#: celery task config
CELERY_TASK_ALWAYS_EAGER = strtobool(env('CELERY_ALWAYS_EAGER', 'false'))
CELERY_TASK_SERIALIZER = 'json'
CELERY_TASK_PROTOCOL = 1
CELERY_TASK_IGNORE_RESULT = True
CELERY_TASK_SEND_EVENTS = False

#: celery worker config
CELERY_WORKER_DISABLE_RATE_LIMITS = True
CELERY_WORKER_TASK_SOFT_TIME_LIMIT = 300
CELERY_WORKER_LOG_FORMAT = '%(message)s level=%(levelname)s process=%(processName)s'
CELERY_WORKER_TASK_LOG_FORMAT = ' '.join([CELERY_WORKER_LOG_FORMAT, 'task=%(task_name)s task_id=%(task_id)s'])
CELERY_WORKER_CONCURRENCY = env('CELERY_WORKER_CONCURRENCY') or None

#: celery routing config
CELERY_TASK_DEFAULT_QUEUE = celery_queue('default')
CELERY_TASK_DEFAULT_EXCHANGE = celery_queue('default')
CELERY_TASK_DEFAULT_ROUTING_KEY = 'default'

CELERY_TASK_QUEUES = (
    Queue(celery_queue('default'), Exchange(celery_queue('default')), routing_key='default'),
    Queue(celery_queue('expiry'), Exchange(celery_queue('expiry'), type='topic'), routing_key='expiry.#'),
    Queue(celery_queue('legal'), Exchange(celery_queue('legal'), type='topic'), routing_key='legal.#'),
    Queue(celery_queue('publish'), Exchange(celery_queue('publish'), type='topic'), routing_key='publish.#'),
)

CELERY_TASK_ROUTES = {
    'apps.archive.content_expiry': {
        'queue': celery_queue('expiry'),
        'routing_key': 'expiry.content'
    },
    'superdesk.io.gc_ingest': {
        'queue': celery_queue('expiry'),
        'routing_key': 'expiry.ingest'
    },
    'superdesk.audit.gc_audit': {
        'queue': celery_queue('expiry'),
        'routing_key': 'expiry.audit'
    },
    'apps.auth.session_purge': {
        'queue': celery_queue('expiry'),
        'routing_key': 'expiry.session'
    },
    'superdesk.commands.remove_exported_files': {
        'queue': celery_queue('expiry'),
        'routing_key': 'expiry.temp_files'
    },
    'content_api.commands.item_expiry': {
        'queue': celery_queue('expiry'),
        'routing_key': 'expiry.content_api'
    },
    'apps.legal_archive.import_legal_publish_queue': {
        'queue': celery_queue('legal'),
        'routing_key': 'legal.publish_queue'
    },
    'apps.legal_archive.commands.import_into_legal_archive': {
        'queue': celery_queue('legal'),
        'routing_key': 'legal.archive'
    },
    'superdesk.publish.transmit': {
        'queue': celery_queue('publish'),
        'routing_key': 'publish.transmit'
    },
    'superdesk.publish.publish_content.publish': {
        'queue': celery_queue('publish'),
        'routing_key': 'publish.transmit'
    },
    'superdesk.publish.publish_content.transmit_subscriber_items': {
        'queue': celery_queue('publish'),
        'routing_key': 'publish.transmit'
    },
    'apps.publish.enqueue.enqueue_published': {
        'queue': celery_queue('publish'),
        'routing_key': 'publish.enqueue'
    },
    'apps.legal_archive.import_legal_archive': {
        'queue': celery_queue('legal'),
        'routing_key': 'legal.archive'
    }
}

#: celery beat config
CELERY_BEAT_SCHEDULE_FILENAME = env('CELERYBEAT_SCHEDULE_FILENAME', './celerybeatschedule.db')
CELERY_BEAT_SCHEDULE = {
    'ingest:update': {
        'task': 'superdesk.io.update_ingest',
        # there is internal schedule for updates per provider,
        # so this is minimal interval when an update can occur
        'schedule': timedelta(seconds=30),
        'options': {'expires': 29}
    },
    'ingest:gc': {
        'task': 'superdesk.io.gc_ingest',
        'schedule': timedelta(minutes=5),
    },
    'audit:gc': {
        'task': 'superdesk.audit.gc_audit',
        'schedule': crontab(minute='0', hour=local_to_utc_hour(1))
    },
    'session:gc': {
        'task': 'apps.auth.session_purge',
        'schedule': timedelta(minutes=5)
    },
    'content:gc': {
        'task': 'apps.archive.content_expiry',
        'schedule': crontab(minute='*/30')
    },
    'temp_files:gc': {
        'task': 'superdesk.commands.temp_file_expiry',
        'schedule': crontab(minute='0', hour=local_to_utc_hour(3))
    },
    'content_api:gc': {
        'task': 'content_api.commands.item_expiry',
        'schedule': crontab(minute='0', hour=local_to_utc_hour(2))
    },
    'publish:transmit': {
        'task': 'superdesk.publish.transmit',
        'schedule': timedelta(seconds=10)
    },
    'content:schedule': {
        'task': 'apps.templates.content_templates.create_scheduled_content',
        'schedule': crontab(minute='*/5'),
    },
    'legal:import_publish_queue': {
        'task': 'apps.legal_archive.import_legal_publish_queue',
        'schedule': timedelta(minutes=5)
    },
    'publish:enqueue': {
        'task': 'apps.publish.enqueue.enqueue_published',
        'schedule': timedelta(seconds=10)
    },
    'legal:import_legal_archive': {
        'task': 'apps.legal_archive.import_legal_archive',
        'schedule': crontab(minute=30, hour=local_to_utc_hour(0))
    },
    'saved_searches:report': {
        'task': 'apps.saved_searches.report',
        'schedule': timedelta(minutes=1)
    },
}

#: Sentry DSN - will report exceptions there
SENTRY_DSN = env('SENTRY_DSN')
SENTRY_INCLUDE_PATHS = ['superdesk', 'apps']

CORE_APPS = [
    'apps.auth',
    'superdesk.roles',
    'superdesk.storage',
    'superdesk.allowed_values',
    'apps.picture_crop',
    'apps.picture_renditions',
    'content_api.publish',
    'content_api.items',
    'content_api.tokens',
    'content_api.items_versions',
    'content_api.api_audit',
    'content_api.search',
    'superdesk.backend_meta',
    'superdesk.server_config',
    'superdesk.internal_destinations',
    'apps.client_config',
    'superdesk.auth',
    'superdesk.attachments',
]

#: Specify what modules should be enabled
INSTALLED_APPS = []

#: LDAP Server (eg: ldap://sourcefabric.org)
LDAP_SERVER = env('LDAP_SERVER', '')
#: LDAP Server port
LDAP_SERVER_PORT = int(env('LDAP_SERVER_PORT', 389))

#: Fully Qualified Domain Name. Ex: sourcefabric.org
LDAP_FQDN = env('LDAP_FQDN', '')

#: LDAP_BASE_FILTER limit the base filter to the security group. Ex: OU=Superdesk Users,dc=sourcefabric,dc=org
LDAP_BASE_FILTER = env('LDAP_BASE_FILTER', '')

#: change the user depending on the LDAP directory structure
LDAP_USER_FILTER = env('LDAP_USER_FILTER', "(&(objectCategory=user)(objectClass=user)(sAMAccountName={}))")

#: LDAP User Attributes to fetch. Keys would be LDAP Attribute Name and Value would be Supderdesk Model Attribute Name
LDAP_USER_ATTRIBUTES = json.loads(env('LDAP_USER_ATTRIBUTES',
                                      '{"givenName": "first_name", "sn": "last_name", '
                                      '"displayName": "display_name", "mail": "email", '
                                      '"ipPhone": "phone"}'))

if LDAP_SERVER:
    CORE_APPS.append('apps.ldap')
else:
    CORE_APPS.append('superdesk.users')
    CORE_APPS.append('apps.auth.db')
    CORE_APPS.append('apps.auth.xmpp')


CORE_APPS.extend([
    'superdesk.upload',
    'superdesk.download',
    'superdesk.sequences',
    'superdesk.notification',
    'superdesk.data_updates',
    'superdesk.activity',
    'superdesk.audit',
    'superdesk.vocabularies',
    'apps.comments',
    'superdesk.profiling',

    'superdesk.io',
    'superdesk.io.feeding_services',
    'superdesk.io.feed_parsers',
    'superdesk.io.webhooks',
    'superdesk.io.subjectcodes',
    'superdesk.io.format_document_for_preview',
    'superdesk.io.iptc',
    'apps.io',
    'apps.io.feeding_services',
    'superdesk.publish',
    'superdesk.commands',
    'superdesk.locators',
    'superdesk.media',

    'apps.auth',
    'apps.archive',
    'apps.archive.item_comments',
    'apps.archive_history',
    'apps.stages',
    'apps.desks',
    'apps.tasks',
    'apps.preferences',
    'apps.spikes',
    'apps.prepopulate',
    'apps.legal_archive',
    'apps.search',
    'apps.saved_searches',
    'apps.suggestions',
    'apps.privilege',
    'apps.rules',
    'apps.highlights',
    'apps.marked_desks',
    'apps.products',
    'apps.publish',
    'apps.export',
    'apps.publish.formatters',
    'apps.content_filters',
    'apps.content_types',
    'apps.dictionaries',
    'apps.duplication',
    'apps.spellcheck',
    'apps.templates',
    'apps.archived',
    'apps.validators',
    'apps.validate',
    'apps.workspace',
    'apps.macros',
    'apps.archive_broadcast',
    'apps.search_providers',
    'apps.feature_preview',
    'apps.workqueue',
    'apps.contacts',
    'apps.monitoring',
    'superdesk.places',
])

RESOURCE_METHODS = ['GET', 'POST']
ITEM_METHODS = ['GET', 'PATCH', 'PUT', 'DELETE']
EXTENDED_MEDIA_INFO = ['content_type', 'name', 'length', '_id']
RETURN_MEDIA_AS_BASE64_STRING = False
VERSION = '_current_version'

#: uses for generation of media url ``(<media_prefix>/<media_id>)``::
MEDIA_PREFIX = env('MEDIA_PREFIX', '%s/upload-raw' % SERVER_URL.rstrip('/'))
MEDIA_PREFIXES_TO_FIX = None

#: amazon access key
AMAZON_ACCESS_KEY_ID = env('AMAZON_ACCESS_KEY_ID', '')
#: amazon secret access key
AMAZON_SECRET_ACCESS_KEY = env('AMAZON_SECRET_ACCESS_KEY', '')
#: amazon region
AMAZON_REGION = env('AMAZON_REGION', 'us-east-1')
#: amazon bucket name
AMAZON_CONTAINER_NAME = env('AMAZON_CONTAINER_NAME', '')
#: use subfolder in bucket to store files
AMAZON_S3_SUBFOLDER = env('AMAZON_S3_SUBFOLDER', '')
#: adds ACL when putting to S3, can be set to ``public-read``, etc.
AMAZON_OBJECT_ACL = env('AMAZON_OBJECT_ACL', '')

RENDITIONS = {
    'picture': {
        'thumbnail': {'width': 220, 'height': 120},
        'viewImage': {'width': 640, 'height': 640},
        'baseImage': {'width': 1400, 'height': 1400},
    },
    'avatar': {
        'thumbnail': {'width': 60, 'height': 60},
        'viewImage': {'width': 200, 'height': 200},
    }
}


#: BCRYPT work factor
BCRYPT_GENSALT_WORK_FACTOR = 12

#: The number of days a token is valid, env ``RESET_PASS_TTL``
RESET_PASSWORD_TOKEN_TIME_TO_LIVE = int(env('RESET_PASS_TTL', 1))
#: The number of days an activation token is valid, env ``ACTIVATE_TTL``
ACTIVATE_ACCOUNT_TOKEN_TIME_TO_LIVE = int(env('ACTIVATE_TTL', 7))

#: email server url
MAIL_SERVER = env('MAIL_SERVER', 'localhost')
#: email server port
MAIL_PORT = int(env('MAIL_PORT', 25))
#: use tls connection
MAIL_USE_TLS = strtobool(env('MAIL_USE_TLS', 'False'))
#: use ssl connection
MAIL_USE_SSL = strtobool(env('MAIL_USE_SSL', 'False'))
#: email account username
MAIL_USERNAME = env('MAIL_USERNAME', '')
#: email account password
MAIL_PASSWORD = env('MAIL_PASSWORD', '')
#: default sender for superdesk emails
MAIL_DEFAULT_SENDER = MAIL_USERNAME or 'superdesk@localhost'

_MAIL_FROM = env('MAIL_FROM', MAIL_USERNAME)

#: list of admin emails - get error notifications by default
ADMINS = [_MAIL_FROM]

DEBUG = strtobool(env('SUPERDESK_DEBUG', 'false'))
SUPERDESK_TESTING = strtobool(env('SUPERDESK_TESTING', 'false'))

#: Set the timezone celery functions to UTC to avoid daylight savings issues SDESK-1057
CELERY_TIMEZONE = 'UTC'

#: The number of days after the user has to change his password
PASSWORD_EXPIRY_DAYS = int(env('PASSWORD_EXPIRY_DAYS', 0))

#: The number of minutes since the last update of the Mongo auth object after which it will be deleted
SESSION_EXPIRY_MINUTES = int(env('SESSION_EXPIRY_MINUTES', 240))

#: The number of minutes before content items are purged
CONTENT_EXPIRY_MINUTES = int(env('CONTENT_EXPIRY_MINUTES', 0))

#: The number of minutes before ingest items are purged
INGEST_EXPIRY_MINUTES = int(env('INGEST_EXPIRY_MINUTES', 2 * 24 * 60))

#: The number of minutes before published content items are purged
PUBLISHED_CONTENT_EXPIRY_MINUTES = int(env('PUBLISHED_CONTENT_EXPIRY_MINUTES', 0))

#: The number of minutes before audit content is purged
AUDIT_EXPIRY_MINUTES = int(env('AUDIT_EXPIRY_MINUTES', 0))

#: The number records to be fetched for expiry.
MAX_EXPIRY_QUERY_LIMIT = int(env('MAX_EXPIRY_QUERY_LIMIT', 100))

#: The number of minutes before Publish Queue is purged
PUBLISH_QUEUE_EXPIRY_MINUTES = int(env('PUBLISH_QUEUE_EXPIRY_MINUTES', 0))

# This setting can be used to apply a limit on the elastic search queries, it is a limit per shard.
# A value of -1 indicates that no limit will be applied.
# If for example the elastic has 5 shards and you wish to limit the number of search results to 1000 then set the value
# to 200 (1000/5).
MAX_SEARCH_DEPTH = int(env('MAX_SEARCH_DEPTH', -1))

#: Defines the maximum value of Ingest Sequence Number after which the value will start from 1
MAX_VALUE_OF_INGEST_SEQUENCE = int(env('MAX_VALUE_OF_INGEST_SEQUENCE', 9999))

DAYS_TO_KEEP = int(env('INGEST_ARTICLES_TTL', '2'))

MACROS_MODULE = env('MACROS_MODULE', 'superdesk.macros')

WS_HOST = env('WSHOST', '0.0.0.0')
WS_PORT = env('WSPORT', '5100')

# Used by the  Kombu Connection. Only valid for the AMQP protocol
WS_HEART_BEAT = int(env('WS_HEARTBEAT', '0'))

#: Defines the maximum value of Publish Sequence Number after which the value will start from 1
MAX_VALUE_OF_PUBLISH_SEQUENCE = int(env('MAX_VALUE_OF_PUBLISH_SEQUENCE', 9999))

#: Defines default value for Source to be set for manually created articles
DEFAULT_SOURCE_VALUE_FOR_MANUAL_ARTICLES = env('DEFAULT_SOURCE_VALUE_FOR_MANUAL_ARTICLES', 'Superdesk')

#: Defines default value for Priority to be set for manually created articles
DEFAULT_PRIORITY_VALUE_FOR_MANUAL_ARTICLES = int(env('DEFAULT_PRIORITY_VALUE_FOR_MANUAL_ARTICLES', 6))

#: Defines default value for Urgency to be set for manually created articles
DEFAULT_URGENCY_VALUE_FOR_MANUAL_ARTICLES = int(env('DEFAULT_URGENCY_VALUE_FOR_MANUAL_ARTICLES', 3))

#: Defines default value for genre to be set for manually created articles
DEFAULT_GENRE_VALUE_FOR_MANUAL_ARTICLES = env('DEFAULT_GENRE_VALUE_FOR_MANUAL_ARTICLES',
                                              [{'qcode': 'Article', 'name': 'Article (news)'}])

#: Defines default value for Priority to be set for ingested articles
DEFAULT_PRIORITY_VALUE_FOR_INGESTED_ARTICLES = int(env('DEFAULT_PRIORITY_VALUE_FOR_INGESTED_ARTICLES', 6))

#: Defines default value for Urgency to be set for ingested articles
DEFAULT_URGENCY_VALUE_FOR_INGESTED_ARTICLES = int(env('DEFAULT_URGENCY_VALUE_FOR_INGESTED_ARTICLES', 3))

#: Defines default value for Priority to be reset for update articles SD-4595
RESET_PRIORITY_VALUE_FOR_UPDATE_ARTICLES = strtobool(env('RESET_PRIORITY_VALUE_FOR_UPDATE_ARTICLES', 'False'))

#: Determines if the ODBC publishing mechanism will be used, If enabled then pyodbc must be installed along with it's
#: dependencies
ODBC_PUBLISH = env('ODBC_PUBLISH', None)

# ODBC test server connection string
ODBC_TEST_CONNECTION_STRING = env('ODBC_TEST_CONNECTION_STRING',
                                  'DRIVER=FreeTDS;DSN=NEWSDB;UID=???;PWD=???;DATABASE=News')

#: This value gets injected into NewsML 1.2 and G2 output documents.
NEWSML_PROVIDER_ID = env('NEWSML_PROVIDER_ID', 'sourcefabric.org')
#: This value gets injected into NewsML 1.2 and G2 output documents.
ORGANIZATION_NAME = env('ORGANIZATION_NAME', 'Your organisation')
#: This value gets injected into NewsML 1.2 and G2 output documents.
ORGANIZATION_NAME_ABBREVIATION = env('ORGANIZATION_NAME_ABBREVIATION', 'Short name for your organisation')

#: max retries when transmitting an item
MAX_TRANSMIT_RETRY_ATTEMPT = int(env('MAX_TRANSMIT_RETRY_ATTEMPT', 10))

#: delay between retry attempts
TRANSMIT_RETRY_ATTEMPT_DELAY_MINUTES = int(env('TRANSMIT_RETRY_ATTEMPT_DELAY_MINUTES', 3))

#: max transmit items to be fetched from mongo at once
MAX_TRANSMIT_QUERY_LIMIT = int(env('MAX_TRANSMIT_QUERY_LIMIT', 500))

#: Code profiling for performance analysis
ENABLE_PROFILING = False

#: default timeout for ftp connections
FTP_TIMEOUT = 300

#: default timeout for email connections
EMAIL_TIMEOUT = 10

#: This setting is used to overide the desk/stage expiry for items when spiked
SPIKE_EXPIRY_MINUTES = None

SECRET_KEY = env('SECRET_KEY', '')

#: secure login
XMPP_AUTH_URL = env('XMPP_AUTH_URL', '')
XMPP_AUTH_DOMAIN = env('XMPP_AUTH_DOMAIN', 'Superdesk')

#: copies basic metadata from parent of associated items
COPY_METADATA_FROM_PARENT = strtobool(env('COPY_METADATA_FROM_PARENT', 'false'))

#: The number of hours before temporary media files are purged
TEMP_FILE_EXPIRY_HOURS = int(env('TEMP_FILE_EXPIRY_HOURS', 24))

#: The number of days before content api items are removed. Defaults to 0 which means no purging occurs
CONTENT_API_EXPIRY_DAYS = int(env('CONTENT_API_EXPIRY_DAYS', 0))

# Google OAuth settings
GOOGLE_CLIENT_ID = env('GOOGLE_CLIENT_ID')
GOOGLE_CLIENT_SECRET = env('GOOGLE_CLIENT_SECRET')

# SAML Auth settings
SAML_PATH = env('SAML_PATH')
SAML_LABEL = env('SAML_LABEL', 'Single Sign On')

EXCLUDED_VOCABULARY_FIELDS = ['default_categories', 'signal', 'rightsinfo', 'subscriber_types',
                              'crop_sizes', 'desk_types', 'replace_words', 'locators',
                              'product_types', 'author_roles', 'job_titles']


#: allows overriding default schema by type, use picture/composite/text keys
SCHEMA = {}
EDITOR = {}

# editor note overriding
#: set to False if you don't want overriding on correction
OVERRIDE_EDNOTE_FOR_CORRECTIONS = True
#: template to use for overriding, you can use {date} and {slugline} placeholders
#: if not set, a default template will be used
# OVERRIDE_EDNOTE_TEMPLATE = 'Story "{slugline}" corrected on {date}'

#: allowed HTML tag in client
HTML_TAGS_WHITELIST = ('h1', 'h2', 'h3', 'h4', 'h6', 'blockquote', 'pre', 'figure', 'ul', 'ol', 'li', 'div', 'p', 'em',
                       'strong', 'i', 'b')

# Japanese reading speed
JAPANESE_CHARACTERS_PER_MINUTE = 600

PUBLISH_ASSOCIATED_ITEMS = False

# Use content profile for validation when auto-publishing
AUTO_PUBLISH_CONTENT_PROFILE = True

#: controll error notifications globally
ERROR_NOTIFICATIONS = strtobool(env('SUPERDESK_ERROR_NOTIFICATIONS', 'true'))

#: geonames credentials
GEONAMES_USERNAME = env('GEONAMES_USERNAME')

# media required fields
VALIDATOR_MEDIA_METADATA = {
    "headline": {
        "required": True,
    },
    "alt_text": {
        "required": True,
    },
    "archive_description": {
        "required": False,
    },
    "description_text": {
        "required": True,
    },
    "copyrightholder": {
        "required": False,
    },
    "byline": {
        "required": False,
    },
    "usageterms": {
        "required": False,
    },
    "copyrightnotice": {
        "required": False,
    },
}



"""Content related helpers and utils.
"""

from eve.utils import config
from superdesk.notification import push_notification
from apps.auth import get_user


def push_content_notification(items, event='content:update'):
    """Push content:update notification for multiple items.

    It can be also 2 versions of same item in updated handler
    so that we sent event with both old and new desk/stage.

    :param list items: list of items
    :param event: custom event name
    """
    ids = {}
    desks = {}
    stages = {}
    for item in items:
        ids[str(item.get('_id', ''))] = 1
        task = item.get('task', {})
        if task.get('desk'):
            desks[str(task.get('desk', ''))] = 1
        if task.get('stage'):
            stages[str(task.get('stage', ''))] = 1
    user = get_user()
    push_notification(
        event,
        user=str(user.get(config.ID_FIELD, '')),
        items=ids,
        desks=desks,
        stages=stages
    )


def push_item_move_notification(original, doc, event='item:move'):
    """Push item:move notification.

    :param original: original doc
    :param doc: doc after updates
    :param event: event name
    """
    from_task = original.get('task', {})
    to_task = doc.get('task', {})
    user = get_user()
    push_notification(
        event,
        user=str(user.get(config.ID_FIELD, '')),
        item=str(original.get(config.ID_FIELD)),
        item_version=str(original.get(config.VERSION)),
        from_desk=str(from_task.get('desk')),
        from_stage=str(from_task.get('stage')),
        to_desk=str(to_task.get('desk')),
        to_stage=str(to_task.get('stage'))
    )


def push_expired_notification(ids, event='item:expired'):
    """Push item:expired notification.

    :param ids: list of expired item ids
    :param event: event name
    """
    push_notification(
        event,
        items={str(_id): 1 for _id in ids}
    )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.io.feed_parsers import XMLFeedParser
from superdesk.io.registry import register_feed_parser
from superdesk.metadata.item import CONTENT_TYPE, ITEM_TYPE
from superdesk.errors import ParserError
from superdesk import etree as sd_etree
from collections import OrderedDict
from superdesk import text_utils
from superdesk.utc import local_to_utc
import superdesk
import dateutil.parser
import logging

logger = logging.getLogger(__name__)
NS = {'r': 'http://tempuri.org/',
      'a': 'http://schemas.microsoft.com/2003/10/Serialization/Arrays'}


class RitzauFeedParser(XMLFeedParser):
    """
    Feed Parser which can parse Ritzau XML feed
    """

    _subjects_map = None

    NAME = 'ritzau'
    label = "Ritzau feed"
    TIMEZONE = 'Europe/Copenhagen'

    def __init__(self):
        super().__init__()

        self.default_mapping = OrderedDict([
            ('guid', 'NewsID/text()'),
            ('body_html', {'xpath': 'content/text()',
                           'filter': sd_etree.clean_html_str}),
            ('firstcreated', {'xpath': 'PublishDate/text()',
                              'filter': self._publish_date_filter,
                              }),
            ('versioncreated', {'xpath': 'PublishDate/text()',
                                'filter': self._publish_date_filter,
                                }),
            ('headline', {'xpath': 'headline/text()',
                          'default': '',
                          'key_hook': self._set_headline}),
            ('priority', 'Priority/text()'),
            ('keywords', {'xpath': 'strapline/text()',
                          'filter': lambda v: list(filter(None, v.split('/')))}),
            ('abstract', 'subtitle'),
            ('byline', 'origin'),
            ('version', {'xpath': 'version/text()',
                         'filter': int}),
            ('ednote', 'Tilredaktionen/text()'),
            ('subject', {'xpath': 'IPTCList/a:int/text()',
                         'list': True,
                         'filter': self._subject_filter})])

    @property
    def subjects_map(self):
        if self._subjects_map is None:
            voc_subjects = superdesk.get_resource_service('vocabularies').find_one(req=None, _id='subject_custom')
            if voc_subjects is not None:
                self._subjects_map = {i['qcode']: i for i in voc_subjects['items']}
            else:
                self._subjects_map = {}
        return self._subjects_map

    def can_parse(self, xml):
        return xml.tag.endswith('RBNews')

    def parse(self, xml, provider=None):
        item = {ITEM_TYPE: CONTENT_TYPE.TEXT,  # set the default type.
                }
        try:
            self.do_mapping(item, xml, namespaces=NS)
        except Exception as ex:
            raise ParserError.parseMessageError(ex, provider)
        return item

    def get_datetime(self, value):
        return dateutil.parser.parse(value)

    def _subject_filter(self, qcode):
        try:
            subject = self.subjects_map[qcode]
        except KeyError:
            return None
        else:
            if not subject.get('is_active', False):
                return None
            name = subject.get('name', '')

        return {'qcode': qcode, 'name': name, 'scheme': 'subject_custom'}

    def _publish_date_filter(self, date_string):
        local = dateutil.parser.parse(date_string)
        return local_to_utc(self.TIMEZONE, local)

    def _set_headline(self, item, value):
        if not value:
            # if there is no headline, we use first 100 chars of body
            # cf. SDNTB-481
            value = text_utils.get_text(item.get('body_html', ''), 'html')[:100]
        item['headline'] = value


register_feed_parser(RitzauFeedParser.NAME, RitzauFeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import json

from eve.utils import ParsedRequest

import superdesk
import logging
from copy import deepcopy
from superdesk import get_resource_service, config
from superdesk.errors import SuperdeskApiError
from superdesk.media.media_operations import crop_image, process_file_from_stream
from superdesk.upload import url_for_media
from superdesk.metadata.item import CONTENT_TYPE, ITEM_TYPE, MEDIA_TYPES, ASSOCIATIONS
from.renditions import _resize_image


logger = logging.getLogger(__name__)


class CropService():

    crop_sizes = []

    def validate_crop(self, original, updates, crop_name):
        """Validate crop info on media item.

        :param dict original: original item
        :param dict updates: updated renditions
        :param str crop_name: name of the crop
        :param dict doc: crop co-ordinates
        :raises SuperdeskApiError.badRequestError:
            For following conditions:
            1) if type != picture
            2) if renditions are missing in the original image
            3) if original rendition is missing
            4) Crop name is invalid
        """
        # Check if type is picture
        if original[ITEM_TYPE] != CONTENT_TYPE.PICTURE:
            raise SuperdeskApiError.badRequestError(message='Only images can be cropped!')

        # Check if the renditions exists
        if not original.get('renditions'):
            raise SuperdeskApiError.badRequestError(message='Missing renditions!')

        # Check if the original rendition exists
        if not original.get('renditions').get('original'):
            raise SuperdeskApiError.badRequestError(message='Missing original rendition!')

        # Check if the crop name is valid
        crop = self.get_crop_by_name(crop_name)
        crop_data = updates.get('renditions', {}).get(crop_name, {})
        if not crop and 'CropLeft' in crop_data:
            raise SuperdeskApiError.badRequestError(message='Unknown crop name! (name=%s)' % crop_name)

        self._validate_values(crop_data)
        self._validate_poi(original, updates, crop_name)
        self._validate_aspect_ratio(crop, crop_data)

    def _validate_values(self, crop):
        int_fields = ('CropLeft', 'CropTop', 'CropRight', 'CropBottom', 'width', 'height')
        for field in int_fields:
            if field in crop:
                try:
                    crop[field] = int(crop[field])
                except (TypeError, ValueError):
                    raise SuperdeskApiError.badRequestError('Invalid value for %s in renditions' % field)

    def _validate_poi(self, original, updates, crop_name):
        """Validate the crop point of interest in the renditions dictionary for the given crop

        :param dict original: original item
        :param dict updates: updated renditions
        """
        renditions = original.get('renditions', {})
        updated_renditions = updates.get('renditions', {})
        original_image = deepcopy(renditions['original'])
        original_image.update(updated_renditions.get('original', {}))
        if 'poi' in updates:
            if 'x' not in updates['poi'] or 'y' not in updates['poi']:
                del updates['poi']
                return
            poi = updates['poi']
        elif 'poi' not in original:
            return
        else:
            if crop_name not in updated_renditions:
                return
            poi = original['poi']

        crop_data = updated_renditions[crop_name] if crop_name in updated_renditions else renditions[crop_name]
        orig_poi_x = int(original_image['width'] * poi['x'])
        orig_poi_y = int(original_image['height'] * poi['y'])

        if orig_poi_y < crop_data.get('CropTop', 0) \
                or orig_poi_y > crop_data.get('CropBottom', original_image['height']) \
                or orig_poi_x < crop_data.get('CropLeft', 0) \
                or orig_poi_x > crop_data.get('CropRight', original_image['width']):
            raise SuperdeskApiError('Point of interest outside the crop %s limits' % crop_name)

    def _validate_aspect_ratio(self, crop, doc):
        """Checks if the aspect ratio is consistent with one in defined in spec

        :param crop: Spec parameters
        :param doc: Posted parameters
        :raises SuperdeskApiError.badRequestError:
        """
        if 'CropLeft' not in doc:
            return

        width = doc['CropRight'] - doc['CropLeft']
        height = doc['CropBottom'] - doc['CropTop']
        if not (crop.get('width') or crop.get('height') or crop.get('ratio')):
            raise SuperdeskApiError.badRequestError(
                message='Crop data are missing. width, height or ratio need to be defined')
        if crop.get('width') and crop.get('height'):
            expected_crop_width = int(crop['width'])
            expected_crop_height = int(crop['height'])
            if width < expected_crop_width or height < expected_crop_height:
                raise SuperdeskApiError.badRequestError(
                    message='Wrong crop size. Minimum crop size is {}x{}.'.format(crop['width'], crop['height']))
                doc_ratio = round(width / height, 1)
                spec_ratio = round(expected_crop_width / expected_crop_height, 1)
                if doc_ratio != spec_ratio:
                    raise SuperdeskApiError.badRequestError(message='Wrong aspect ratio!')
        elif crop.get('ratio'):
            ratio = crop.get('ratio')
            if type(ratio) not in [int, float]:
                ratio = ratio.split(':')
                ratio = int(ratio[0]) / int(ratio[1])
            if abs((width / height) - ratio) > 0.1:
                raise SuperdeskApiError.badRequestError(
                    message='Ratio %s is not respected. We got %f' % (crop.get('ratio'), abs((width / height))))

    def get_crop_by_name(self, crop_name):
        """Finds the crop in the list of crops by name

        :param crop_name: Crop name
        :return: Matching crop or None
        """
        if not self.crop_sizes:
            self.crop_sizes = get_resource_service('vocabularies').find_one(req=None, _id='crop_sizes').get('items')

        if not self.crop_sizes:
            raise SuperdeskApiError.badRequestError(message='Crops sizes couldn\'t be loaded!')

        return next((c for c in self.crop_sizes if c.get('name', '').lower() == crop_name.lower()), None)

    def create_crop(self, original_image, crop_name, crop_data):
        """Create a new crop based on the crop co-ordinates

        :param original: Article to add the crop
        :param crop_name: Name of the crop
        :param doc: Crop details
        :raises SuperdeskApiError.badRequestError
        :return dict: rendition
        """
        original_file = superdesk.app.media.fetch_rendition(original_image)
        if not original_file:
            raise SuperdeskApiError.badRequestError('Original file couldn\'t be found')
        try:
            cropped, out = crop_image(original_file, crop_name, crop_data)
            crop = self.get_crop_by_name(crop_name)
            if not cropped:
                raise SuperdeskApiError.badRequestError('Saving crop failed.')
            # resize if needed
            if crop.get('width') or crop.get('height'):
                out, width, height = _resize_image(out,
                                                   size=(crop.get('width'), crop.get('height')),
                                                   keepProportions=crop.get('keep_proportions', True))
                crop['width'] = width
                crop['height'] = height
                out.seek(0)
            return self._save_cropped_image(out, original_image, crop_data)
        except SuperdeskApiError:
            raise
        except Exception as ex:
            raise SuperdeskApiError.badRequestError('Generating crop failed: {}'.format(str(ex)))

    def _save_cropped_image(self, file_stream, original, doc):
        """Saves the cropped image and returns the crop dictionary

        :param file_stream: cropped image stream
        :param original: original rendition
        :param doc: crop data
        :return dict: Crop values
        :raises SuperdeskApiError.internalError
        """
        crop = {}
        try:
            file_name, content_type, metadata = process_file_from_stream(file_stream,
                                                                         content_type=original.get('mimetype'))
            file_stream.seek(0)
            file_id = superdesk.app.media.put(file_stream, filename=file_name,
                                              content_type=content_type,
                                              resource='upload',
                                              metadata=metadata)
            crop['media'] = file_id
            crop['mimetype'] = content_type
            crop['href'] = url_for_media(file_id, content_type)
            crop['CropTop'] = doc.get('CropTop', None)
            crop['CropLeft'] = doc.get('CropLeft', None)
            crop['CropRight'] = doc.get('CropRight', None)
            crop['CropBottom'] = doc.get('CropBottom', None)
            return crop
        except Exception as ex:
            try:
                superdesk.app.media.delete(file_id)
            except Exception:
                pass
            raise SuperdeskApiError.internalError('Generating crop failed: {}'.format(str(ex)), exception=ex)

    def _delete_crop_file(self, file_id):
        """Delete the crop file

        :param Object_id file_id: Object_Id of the file.
        """
        try:
            superdesk.app.media.delete(file_id)
        except Exception:
            logger.exception("Crop File cannot be deleted. File_Id {}".format(file_id))

    def create_multiple_crops(self, updates, original):
        """Create multiple crops based on the renditions.

        :param dict updates: update item
        :param dict original: original of the updated item
        """
        if original.get(ITEM_TYPE) != CONTENT_TYPE.PICTURE:
            return

        update_renditions = updates.get('renditions', {})
        renditions = deepcopy(original.get('renditions', {}))
        # keep renditions updates (urls may have changed)
        renditions.update(update_renditions)
        if 'original' in updates.get('renditions', {}):
            original_image = updates['renditions']['original']
        else:
            original_image = original['renditions']['original']

        for key in update_renditions:
            if not self.get_crop_by_name(key):
                continue

            original_crop = original.get('renditions', {}).get(key, {})
            fields = ('CropLeft', 'CropTop', 'CropRight', 'CropBottom')
            crop_data = update_renditions.get(key, {})
            if any(crop_data.get(name) != original_crop.get(name) for name in fields) and not crop_data.get('media'):
                rendition = self.create_crop(original_image, key, crop_data)
                renditions[key] = rendition

        poi = updates.get('poi')
        if poi:
            for crop_name in renditions:
                self._set_crop_poi(renditions, crop_name, poi)

        updates['renditions'] = renditions

    def _set_crop_poi(self, renditions, crop_name, poi):
        """Set the crop point of interest in the renditions dictionary for the given crop

        :param dict renditions: updated renditions
        :param string crop_name: the crop for which to set the poi
        :param dict poi: the point of interest dictionary
        """
        fields = ('CropLeft', 'CropTop', 'CropRight', 'CropBottom')
        if 'x' in poi and 'y' in poi:
            original_image = renditions['original']
            crop_data = renditions[crop_name]
            orig_poi_x = int(original_image['width'] * poi['x'])
            orig_poi_y = int(original_image['height'] * poi['y'])

            if any(name in crop_data for name in fields):
                crop_poi_x = orig_poi_x - crop_data.get('CropLeft', 0)
                crop_poi_y = orig_poi_y - crop_data.get('CropTop', 0)
            else:
                crop_poi_x = int(crop_data.get('width', original_image['width']) * poi['x'])
                crop_poi_y = int(crop_data.get('height', original_image['height']) * poi['y'])
            renditions[crop_name]['poi'] = {'x': crop_poi_x, 'y': crop_poi_y}

    def validate_multiple_crops(self, updates, original):
        """Validate crops for the image

        :param dict updates: update item
        :param dict original: original of the updated item
        """
        renditions = updates.get('renditions', {})
        if not (renditions and original.get(ITEM_TYPE) == CONTENT_TYPE.PICTURE):
            return

        for key in renditions:
            self.validate_crop(original, updates, key)

    def delete_replaced_crop_files(self, updates, original):
        """Delete the replaced crop files.

        :param dict updates: update item
        :param dict original: original of the updated item
        """
        update_renditions = updates.get('renditions', {})
        if original.get(ITEM_TYPE) == CONTENT_TYPE.PICTURE and update_renditions:
            renditions = original.get('renditions', {})
            for key in update_renditions:
                if self.get_crop_by_name(key) and \
                        update_renditions.get(key, {}).get('media') != \
                        renditions.get(key, {}).get('media'):
                    self._delete_crop_file(renditions.get(key, {}).get('media'))

    def update_media_references(self, updates, original, published=False):
        """Update the media references collection.
         When item (media item or associated media) is updated or created,
         media_references are created. These media_references are updated to published state
         once the item is published.

        :param dict updates: Updates of the item
        :param dict original: Original item
        :param boolean published: True if publishing the item else False
        """
        item_id = original.get(config.ID_FIELD)
        references = {}
        if updates.get('renditions', original.get('renditions', {})):
            references = {
                item_id: updates.get('renditions', original.get('renditions', {}))
            }

        if original.get(ITEM_TYPE) not in MEDIA_TYPES:
            associations = updates.get(ASSOCIATIONS) or original.get(ASSOCIATIONS)
            if not associations:
                return

            references = {assoc.get(config.ID_FIELD): assoc.get('renditions')
                          for assoc in associations.values() if assoc and assoc.get('renditions')}

        if not references:
            return

        for assoc_id, renditions in references.items():
            associated_id = assoc_id if assoc_id != item_id else None
            for rendition in renditions.values():
                if not rendition.get('media'):
                    continue

                media = str(rendition.get('media'))
                reference = get_resource_service('media_references').find_one(req=None, item_id=item_id,
                                                                              media_id=media)
                if not reference:
                    try:
                        get_resource_service('media_references').post([{'item_id': item_id,
                                                                        'media_id': media,
                                                                        'associated_id': associated_id,
                                                                        'published': False}])
                    except Exception:
                        logger.exception('Failed to insert media reference item {} media {}'.format(item_id, media))

        # item is publish
        if not published:
            return

        req = ParsedRequest()
        req.where = json.dumps({'item_id': item_id, 'published': False})
        refs = list(get_resource_service('media_references').get(req=req, lookup=None))
        for ref in refs:
            try:
                get_resource_service('media_references').patch(ref.get(config.ID_FIELD),
                                                               updates={'published': True})
            except Exception:
                logger.exception('Failed to update media '
                                 'reference item {} media {}'.format(ref.get("item_id"), ref.get("media_id")))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.media.crop import CropService
from superdesk.metadata.item import ITEM_STATE, EMBARGO, SCHEDULE_SETTINGS
from superdesk.utc import utcnow
from superdesk.text_utils import update_word_count
from apps.archive.common import set_sign_off, ITEM_OPERATION
from apps.archive.archive import update_associations
from .common import BasePublishService, BasePublishResource, ITEM_CORRECT


class CorrectPublishResource(BasePublishResource):

    def __init__(self, endpoint_name, app, service):
        super().__init__(endpoint_name, app, service, 'correct')


class CorrectPublishService(BasePublishService):
    publish_type = 'correct'
    published_state = 'corrected'
    item_operation = ITEM_CORRECT

    def set_state(self, original, updates):
        updates[ITEM_STATE] = self.published_state

        if original.get(EMBARGO) or updates.get(EMBARGO):
            # embargo time elapsed
            utc_embargo = updates.get(SCHEDULE_SETTINGS, {}).get('utc_{}'.format(EMBARGO)) or \
                original.get(SCHEDULE_SETTINGS, {}).get('utc_{}'.format(EMBARGO))
            if utc_embargo and utc_embargo < utcnow():
                # remove embargo information. so the next correction is without embargo.
                updates[EMBARGO] = None
                super().set_state(original, updates)
        else:
            super().set_state(original, updates)

    def on_update(self, updates, original):
        update_associations(updates)
        CropService().validate_multiple_crops(updates, original)
        super().on_update(updates, original)
        updates[ITEM_OPERATION] = self.item_operation
        updates['versioncreated'] = utcnow()
        updates['correction_sequence'] = original.get('correction_sequence', 1) + 1
        set_sign_off(updates, original)
        update_word_count(updates, original)

    def update(self, id, updates, original):
        CropService().create_multiple_crops(updates, original)
        super().update(id, updates, original)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import Resource


class MediaReferencesResource(Resource):
    schema = {
        # item where media belongs i.e. in case of picture item it will be picture item
        # in case of associated featuremedia it will be the parent item id.
        'item_id': {
            'type': 'string',
            'required': True
        },
        # media of the renditions
        'media_id': {
            'type': 'string',
            'required': True
        },
        # associated item id
        'associated_id': {
            'type': 'string',
            'nullable': True
        },
        'published': {
            'type': 'boolean'
        }
    }
    endpoint_name = 'media_references'
    internal_resource = False
    resource_methods = ['GET']
    item_methods = ['GET']
    mongo_indexes = {
        'item_id_1_media_1': [('item_id', 1), ('media_id', 1)],
        'media_1': [('media_id', 1)],
        'associated_id': [('associated_id', 1)]
    }
    query_objectid_as_string = True



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015, 2016 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from apps.validate.validate import SchemaValidator, ValidateService
from superdesk.tests import TestCase
from superdesk.metadata.item import ITEM_TYPE
from superdesk.default_settings import VALIDATOR_MEDIA_METADATA


# dict of fields mandatory in media during validation
# this is build dynamically to stay in sync with VALIDATOR_MEDIA_METADATA
MEDIA_MANDATORY = {k: k for k, v in VALIDATOR_MEDIA_METADATA.items() if v.get("required")}


class ValidateMandatoryInListTest(TestCase):

    def test_fail_validate_mandatory_in_list_for_subject(self):
        validator = SchemaValidator()
        mandatory = {'scheme': {'subject': 'custom_subject', 'category': 'category'}}
        field = 'scheme'
        value = [{'name': 'DiDFødselsdag', 'qcode': 'DiDFødselsdag',
                  'scheme': 'category', 'service': {'d': 1, 'i': 1}}]
        validator._validate_mandatory_in_list(mandatory, field, value)

        self.assertEqual(validator._errors['subject'], 'is a required field')

    def test_fail_validate_mandatory_in_list_for_subject_and_category(self):
        validator = SchemaValidator()
        mandatory = {'scheme': {'subject': 'custom_subject', 'category': 'category'}}
        field = 'scheme'
        value = []
        validator._validate_mandatory_in_list(mandatory, field, value)

        self.assertEqual(validator._errors['subject'], 'is a required field')
        self.assertEqual(validator._errors['category'], 'is a required field')

    def test_validate_mandatory_in_list(self):
        validator = SchemaValidator()
        mandatory = {'scheme': {'subject': 'subject_custom', 'category': 'category'}}
        field = 'scheme'
        value = [{'name': 'DiDFødselsdag', 'qcode': 'DiDFødselsdag',
                  'scheme': 'category', 'service': {'d': 1, 'i': 1}},
                 {'name': 'arkeologi', 'qcode': '01001000', 'scheme': 'subject_custom', 'parent': '01000000'}]
        validator._validate_mandatory_in_list(mandatory, field, value)

        self.assertEqual(validator._errors, {})

    def test_sanitize_fields_not_in_schema(self):
        doc = {'body_html': 'test'}
        service = ValidateService()
        schema = {'schema': {'body_html': None}}
        service._sanitize_fields(doc, schema)
        self.assertEqual('test', doc['body_html'])

    def test_validate_date_with_success(self):
        validator = SchemaValidator()
        validator._validate_type_date('test1', '2017-11-22T22:11:33+0000')
        self.assertEqual(validator._errors, {})

    def test_validate_date_with_error(self):
        validator = SchemaValidator()
        validator._validate_type_date('test1', '2017-11-33T22:11:33+0000')
        self.assertEqual(validator._errors, {'test1': 'require a date value'})

    def test_validate_field_without_schema(self):
        self.app.data.insert('content_types', [{'_id': 'foo', 'schema': {
            'slugline': None,
            'headline': {'required': True},
        }}])
        service = ValidateService()
        errors = service.create([
            {
                'act': 'test',
                'type': 'test',
                'validate': {'profile': 'foo', 'slugline': 'foo'},
            },
        ])
        self.assertEqual(['HEADLINE is a required field'], errors[0])

    def test_validate_required_empty_string(self):
        self.app.data.insert('content_types', [
            {'_id': 'foo', 'schema': {'headline': {'required': True}}}
        ])

        service = ValidateService()
        errors = service.create([
            {'act': 'test', 'type': 'test', 'validate': {'profile': 'foo', 'headline': ''}}
        ])

        self.assertEqual(['HEADLINE empty values not allowed'], errors[0])

    def test_validate_required_empty_list(self):
        self.app.data.insert('content_types', [
            {'_id': 'foo', 'schema': {'subject': {'type': 'list', 'required': True}}}
        ])

        service = ValidateService()
        errors = service.create([
            {'act': 'test', 'type': 'test', 'validate': {'profile': 'foo', 'subject': []}}
        ])

        self.assertEqual(errors, [['SUBJECT is a required field']])

    def test_validate_required_none_list(self):
        self.app.data.insert('content_types', [{
            '_id': 'foo',
            'schema': {
                'subject': {
                    'type': 'list',
                    'required': True,
                    'mandatory_in_list': {'scheme': {'subject': 'subject_custom', 'category': 'category'}},
                    'schema': {
                        'type': 'dict',
                        'schema': {
                            'name': {},
                            'qcode': {},
                            'scheme': {
                                'type': 'string',
                                'required': True,
                                'allowed': ['subject_custom', 'category']
                            },
                            'service': {'nullable': True},
                            'parent': {'nullable': True}
                        }
                    }
                }
            }
        }])

        service = ValidateService()
        errors = service.create([
            {'act': 'test', 'type': 'test', 'validate': {'profile': 'foo', 'subject': None}}
        ])

        self.assertEqual(errors, [['SUBJECT is a required field']])

    def test_validate_field_required_feature_media(self):
        self.app.data.insert('content_types', [{'_id': 'foo', 'schema': {
            'slugline': None,
            'feature_media': {'required': True},
        }}])
        service = ValidateService()
        errors = service.create([
            {
                'act': 'test',
                'type': 'test',
                'validate': {'profile': 'foo', 'slugline': 'foo'},
            },
        ])
        self.assertEqual(['FEATURE_MEDIA is a required field'], errors[0])

    def test_validate_field_required_media_description_empty(self):
        self.app.data.insert('content_types', [{'_id': 'foo', 'schema': {
            'slugline': None,
            'feature_media': {'required': True, 'type': 'media'},
            'media_description': {'required': True}
        }}])
        service = ValidateService()
        errors = service.create([
            {
                'act': 'test',
                'type': 'test',
                'validate': {'profile': 'foo', 'slugline': 'foo', 'associations': {'featuremedia': {}}},
            }
        ])
        self.assertIn('MEDIA_DESCRIPTION is a required field', errors[0])

    def test_validate_field_required_media_description_null(self):
        self.app.data.insert('content_types', [{'_id': 'foo', 'schema': {
            'slugline': None,
            'feature_media': {'required': True},
            'media_description': {'required': True},
        }}])
        service = ValidateService()
        errors = service.create([
            {
                'act': 'test',
                'type': 'test',
                'validate': {'profile': 'foo', 'slugline': 'foo', 'associations': {'featuremedia': None}},
            },
        ])
        self.assertIn('FEATURE_MEDIA is a required field', errors[0])
        self.assertIn('MEDIA_DESCRIPTION is a required field', errors[0])

    def test_validate_field_required_media_description_required_false(self):
        self.app.data.insert('content_types', [{'_id': 'foo', 'schema': {
            'slugline': None,
            'feature_media': {'required': True, 'type': 'media'},
            'media_description': {'required': False}
        }}])
        service = ValidateService()
        errors = service.create([
            {
                'act': 'test',
                'type': 'test',
                'validate': {'profile': 'foo', 'slugline': 'foo', 'associations': {'featuremedia': None}},
            },
        ])

        self.assertIn('FEATURE_MEDIA is a required field', errors[0])

    def test_validate_field_required_media_description_required_false_null_true(self):
        self.app.data.insert('content_types', [{'_id': 'foo', 'schema': {
            'slugline': None,
            'feature_media': {'required': False, 'nullable': True, 'type': 'media'},
            'media_description': {'required': False, 'nullable': True},
        }}])
        service = ValidateService()
        errors = service.create([
            {
                'act': 'test',
                'type': 'test',
                'validate': {'profile': 'foo', 'slugline': 'foo', 'associations': {'featuremedia': None}},
            },
        ])

        self.assertEqual([], errors[0])

    def test_validate_field_feature_media_and_media_description(self):
        self.app.data.insert('content_types', [{'_id': 'foo', 'schema': {
            'slugline': None,
            'feature_media': {'required': True, 'type': 'media'},
            'media_description': {'required': True},
        }}])
        service = ValidateService()
        feature_media = MEDIA_MANDATORY
        feature_media.update({'description_text': 'test'})
        errors = service.create([
            {
                'act': 'test',
                'type': 'test',
                'validate': {
                    'profile': 'foo',
                    'slugline': 'foo',
                    'associations': {'featuremedia': feature_media}
                },
            },
        ])
        self.assertEqual(errors, [[]])

    def test_validate_custom_fields(self):
        self.app.data.insert('content_types', [{'_id': 'foo', 'schema': {
            'embed1': {
                "required": True,
                "enabled": True,
                "type": "embed",
                "nullable": False
            },
            'text1': {
                "minlength": 10,
                "required": True,
                "enabled": True,
                "type": "text",
                "maxlength": 160,
                "nullable": False
            },
            'date1': {
                "required": True,
                "enabled": True,
                "type": "date",
                "nullable": False
            }
        }}])
        self.app.data.insert('vocabularies', [
            {'_id': 'embed1', 'type': 'manageable', 'field_type': 'embed'},
            {'_id': 'text1', 'type': 'manageable', 'field_type': 'text'},
            {'_id': 'date1', 'type': 'manageable', 'field_type': 'date'}
        ])
        service = ValidateService()
        doc = {
            'act': 'test',
            ITEM_TYPE: 'test',
            'validate': {
                'profile': 'foo'
            }
        }
        schema = {'extra': {
            'schema': {
                'embed1': {
                    'required': True,
                    'enabled': True,
                    'nullable': False,
                    'empty': False,
                    'type': 'dict'
                },
                'text1': {
                    'enabled': True,
                    'required': True,
                    'nullable': False,
                    'minlength': 10,
                    'maxlength': 160,
                    'type': 'string'
                },
                'date1': {
                    'required': True,
                    'enabled': True,
                    'nullable': False,
                    'empty': False,
                    'type': 'date'
                },
            },
            'type': 'dict'
        }}
        self.assertEqual(service._get_validators(doc)[0]['schema'], schema)

    def test_sanitize_text_fields(self):
        item = {
            'headline': '<p>headline</p>',
            'extra': {'text1': '<p>text 1</p>'}
        }
        sanitized_item = {
            'headline': 'headline',
            'extra': {'text1': 'text 1'}
        }
        validator = {
            'schema': {
                'headline': {
                    'maxlength': 64, 'nullable': True, 'required': False, 'type': 'string'
                },
                'extra': {
                    'schema': {
                        'text1': {
                            'maxlength': 10, 'nullable': True, 'required': False, 'type': 'string'
                        }
                    }
                }
            }
        }
        ValidateService()._sanitize_fields(item, validator)
        self.assertEqual(item, sanitized_item)

    def test_validate_field_sms(self):
        self.app.data.insert('content_types', [{'_id': 'foo', 'schema': {
            'sms': {
                "minlength": 10,
                "required": True,
                "enabled": True,
                "type": "string",
                "maxlength": 160,
                "nullable": True
            }}}])
        service = ValidateService()
        errors = service.create([
            {
                'act': 'test',
                'type': 'test',
                'validate': {'profile': 'foo',
                             'flags': {'marked_for_sms': True},
                             'sms_message': 'short'
                             },
            },
        ])
        self.assertEqual(['SMS is too short'], errors[0])

    def test_validate_field_sms_not_enabled(self):
        self.app.data.insert('content_types', [{'_id': 'foo', 'schema': {
            'sms': {
                "minlength": 10,
                "required": True,
                "enabled": True,
                "type": "string",
                "maxlength": 160,
                "nullable": True
            }}}])
        service = ValidateService()
        errors = service.create([
            {
                'act': 'test',
                'type': 'test',
                'validate': {'profile': 'foo',
                             'flags': {'marked_for_sms': False},
                             'sms_message': 'short'
                             },
            },
        ])
        self.assertEqual(errors, [[]])

    def test_validate_process_media(self):
        item = {
            'associations': {
                'media1--1': {
                    'headline': 'media 1'
                }
            }
        }
        validation_schema = {
            'media1': {'required': True}
        }
        ValidateService()._process_media(item, validation_schema)
        self.assertIn('media1', item)
        self.assertEqual(item['associations']['media1--1'], item['media1'])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk.io.feed_parsers.newsml_2_0 import NewsMLTwoFeedParser
from superdesk.tests import TestCase
from superdesk.etree import etree
import os


class BaseAPTestCase(TestCase):

    vocab = [{'_id': 'genre', 'items': [{'name': 'Current'}]}]

    def setUp(self):
        with self.app.app_context():
            self.app.data.insert('vocabularies', self.vocab)
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', self.filename))
        provider = {'name': 'Test'}
        with open(fixture, 'rb') as f:
            self.root_elt = etree.fromstring(f.read())
            self.items = NewsMLTwoFeedParser().parse(self.root_elt, provider)


class APTestCase(BaseAPTestCase):
    filename = 'ap.xml'

    def test_can_parse(self):
        self.assertTrue(NewsMLTwoFeedParser().can_parse(self.root_elt))

    def test_content(self):
        item = self.items[0]
        self.assertEqual(item['version'], '2')
        self.assertEqual(item['headline'], 'Nutrisystem beats 4Q earnings and revenue expectations')
        self.assertEqual(item['slugline'], 'BC-US--Earns-Nutrisystem, 1st Ld-Writethru')
        self.assertEqual(item['body_html'], '<p>FORT WASHINGTON, Pa. (AP) _ Nutrisystem Inc. (NTRI) on Monday reported'
                                            ' fourth-quarter net income of $10.9 million.</p>\n              \n<p>On a'
                                            ' per-share basis, the Fort Washington, Pennsylvania-based company said it'
                                            ' had profit of 36 cents. Earnings, adjusted for non-recurring costs, were'
                                            ' 42 cents per share.</p>\n              \n<p>The results beat Wall Street'
                                            ' expectations. The average estimate of four analysts surveyed by Zacks In'
                                            'vestment Research was for earnings of 41 cents per share.</p>\n          '
                                            '    \n<p>The weight-loss company posted revenue of $131.2 million in the '
                                            'period, also exceeding Street forecasts. Four analysts surveyed by Zacks '
                                            'expected $129.1 million.</p>\n              \n<p>For the year, the compan'
                                            'y reported profit of $57.9 million, or $1.90 per share. Revenue was repor'
                                            'ted as $697 million.</p>\n              \n<p>For the current quarter endi'
                                            'ng in April, Nutrisystem said it expects revenue in the range of $204 mil'
                                            'lion to $209 million.</p>\n              \n<p>The company expects full-ye'
                                            'ar earnings to be $1.99 to $2.09 per share, with revenue ranging from $68'
                                            '5 million to $705 million.</p>\n              \n<p>Nutrisystem shares hav'
                                            'e decreased 24 percent since the beginning of the year. In the final minu'
                                            'tes of trading on Monday, shares hit $39.90, a climb of almost 5 percent '
                                            'in the last 12 months.</p>\n              \n<p>_____</p>\n              '
                                            '\n<p>This story was generated by Automated Insights (http://automatedinsig'
                                            'hts.com/ap) using data from Zacks Investment Research. Access a Zacks sto'
                                            'ck report on NTRI at https://www.zacks.com/ap/NTRI</p>\n            ')
        self.assertEqual(item['guid'], 'tag:ap.org,2011:5e565888b587889c5c8bd62b6164118c-text:2')
        self.assertEqual(item['firstcreated'].isoformat(), '2018-02-26T21:20:36+00:00')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk

from eve.io.base import DataLayer
from eve.io.mongo import Mongo
from eve.utils import config, ParsedRequest
from eve_elastic import Elastic, InvalidSearchString  # noqa
from flask import current_app
from superdesk.lock import lock, unlock
from superdesk.json_utils import SuperdeskJSONEncoder


class SuperdeskDataLayer(DataLayer):
    """Superdesk Data Layer.

    Implements eve data layer interface, is used to make eve work with superdesk service layer.
    It handles app initialization and later it forwards eve calls to respective service.
    """

    serializers = {}
    serializers.update(Mongo.serializers)
    serializers.update({'datetime': Elastic.serializers['datetime']})

    def init_app(self, app):
        app.data = self  # app.data must be set for locks to work
        self.mongo = Mongo(app)
        self.driver = self.mongo.driver
        self.storage = self.driver
        self.elastic = Elastic(app, serializer=SuperdeskJSONEncoder(), skip_index_init=True, retry_on_timeout=True)

    def pymongo(self, resource=None, prefix=None):
        return self.mongo.pymongo(resource, prefix)

    def init_elastic(self, app):
        """Init elastic index.

        It will create index and put mapping. It should run only once so locks are in place.
        Thus mongo must be already setup before running this.
        """
        with app.app_context():
            if lock('elastic', expire=10):
                try:
                    self.elastic.init_index(app)
                finally:
                    unlock('elastic')

    def find(self, resource, req, lookup):
        return superdesk.get_resource_service(resource).get(req=req, lookup=lookup)

    def find_all(self, resource, max_results=1000):
        req = ParsedRequest()
        req.max_results = max_results
        return self._backend(resource).find(resource, req, None)

    def find_one(self, resource, req, **lookup):
        item = superdesk.get_resource_service(resource).find_one(req=req, **lookup)
        if item is not None:
            item.setdefault('_type', self.datasource(resource)[0])
        return item

    def find_one_raw(self, resource, _id):
        return self._backend(resource).find_one_raw(resource, _id)

    def find_list_of_ids(self, resource, ids, client_projection=None):
        return self._backend(resource).find_list_of_ids(resource, ids, client_projection)

    def insert(self, resource, docs, **kwargs):
        return superdesk.get_resource_service(resource).create(docs, **kwargs)

    def update(self, resource, id_, updates, original):
        return superdesk.get_resource_service(resource).update(id=id_, updates=updates, original=original)

    def update_all(self, resource, query, updates):
        datasource = self.datasource(resource)
        driver = self._backend(resource).driver
        collection = driver.db[datasource[0]]
        return collection.update(query, {'$set': updates}, multi=True)

    def replace(self, resource, id_, document, original):
        return superdesk.get_resource_service(resource).replace(id=id_, document=document, original=original)

    def remove(self, resource, lookup=None):
        if lookup is None:
            lookup = {}
        return superdesk.get_resource_service(resource).delete(lookup=lookup)

    def is_empty(self, resource):
        return self._backend(resource).is_empty(resource)

    def _search_backend(self, resource):
        if resource.endswith(current_app.config['VERSIONS']):
            return
        datasource = self.datasource(resource)
        backend = config.SOURCES.get(datasource[0], {}).get('search_backend', None)
        return getattr(self, backend) if backend is not None else None

    def _backend(self, resource):
        datasource = self.datasource(resource)
        backend = config.SOURCES.get(datasource[0], {'backend': 'mongo'}).get('backend', 'mongo')
        return getattr(self, backend)

    def get_mongo_collection(self, resource):
        return self.mongo.pymongo(resource).db[resource]

    def get_elastic_resources(self):
        """Get set of available elastic resources."""
        resources = set()
        for resource in config.SOURCES:
            datasource = self.datasource(resource)[0]
            if self._search_backend(datasource):
                resources.add(datasource)
        return resources



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import logging

from superdesk.io.registry import register_feeding_service, register_feeding_service_parser
from superdesk.io.feeding_services import FeedingService
from superdesk.errors import IngestApiError, SuperdeskIngestError
from lxml import etree
import requests

logger = logging.getLogger(__name__)
URL = 'https://services.ritzau.dk/ritzaurest/Services.svc/xml/news/NewsQueue'
URL_ACK = 'https://services.ritzau.dk/ritzaurest/Services.svc/xml/news/QueueAcknowledge'


class RitzauFeedingService(FeedingService):
    """
    Feeding Service class which can retrieve articles from Ritzau web service
    """

    NAME = 'ritzau'

    ERRORS = [IngestApiError.apiRequestError().get_error_description(),
              SuperdeskIngestError.notConfiguredError().get_error_description()]

    label = 'Ritzau feed API'

    fields = [
        {
            'id': 'username', 'type': 'text', 'label': 'Username',
            'placeholder': 'Username', 'required': True
        },
        {
            'id': 'password', 'type': 'password', 'label': 'Password',
            'placeholder': 'Password', 'required': True
        },
        {
            'id': 'url', 'type': 'text', 'label': 'URL',
            'placeholder': 'fill this field only for advanced uses', 'required': False
        }
    ]

    def _update(self, provider, update):
        try:
            config = provider['config']
            user = config['username']
            password = config['password']
        except KeyError:
            SuperdeskIngestError.notConfiguredError(Exception('username and password are needed'))

        url_override = config.get('url', '').strip()
        if not url_override.startswith('http'):
            SuperdeskIngestError.notConfiguredError(Exception('if URL is set, it must be a valid http link'))

        if url_override:
            params = {'user': user, 'password': password, 'maksAntal': 50}
        else:
            params = {'user': user, 'password': password, 'maksAntal': 50, 'waitAcknowledge': 'true'}

        try:
            r = requests.get(url_override or URL, params=params)
        except Exception:
            raise IngestApiError.apiRequestError(Exception('error while doing the request'))

        try:
            root_elt = etree.fromstring(r.text)
        except Exception:
            raise IngestApiError.apiRequestError(Exception('error while parsing the request answer'))

        try:
            if root_elt.xpath('(//error/text())[1]')[0] != '0':
                err_msg = root_elt.xpath('(//errormsg/text())[1]')[0]
                raise IngestApiError.apiRequestError(Exception('error code returned by API: {msg}'.format(msg=err_msg)))
        except IndexError:
            raise IngestApiError.apiRequestError(Exception('Invalid XML, <error> element not found'))

        parser = self.get_feed_parser(provider)
        items = []
        for elt in root_elt.xpath('//RBNews'):
            item = parser.parse(elt, provider)
            items.append(item)
            if not url_override:
                try:
                    queue_id = elt.xpath('.//ServiceQueueId/text()')[0]
                except IndexError:
                    raise IngestApiError.apiRequestError(Exception('missing ServiceQueueId element'))
                ack_params = {'user': user, 'password': password, 'servicequeueid': queue_id}
                try:
                    requests.get(URL_ACK, params=ack_params)
                except Exception:
                    raise IngestApiError.apiRequestError(Exception('error while doing the request'))

        return [items]


register_feeding_service(RitzauFeedingService)
register_feeding_service_parser(RitzauFeedingService.NAME, 'ritzau')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2016 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import cProfile
import logging
from superdesk.errors import SuperdeskApiError
from superdesk.services import BaseService

from eve.utils import config


logger = logging.getLogger(__name__)

profile = cProfile.Profile()


class Cursor:
    """
    Simulate pymongo Cursor for retuned results
    """

    def __init__(self, iterable):
        self.iterable = iterable

    def __iter__(self):
        return self.iterable.__iter__()

    def __getitem__(self, index):
        return self.iterable[index]

    def __next__(self):
        yield self.iterable.__next__()

    def count(self, with_limit_and_skip=False):
        return len(self.iterable)


class ProfilingService(BaseService):
    """
    Allows reading of the profiling data
    """

    def on_create(self, docs):
        for doc in docs:
            doc[config.ID_FIELD] = doc['name']

    def delete(self, lookup):
        """
        Resets the profiling data.
        """
        try:
            profile.disable()
            from superdesk.profiling import dump_stats
            dump_stats(profile, 'rest')
        finally:
            profile.enable()
        return True

    def _get_sort(self, req):
        """
        Returns the sort fields in a list based on the received request.
        """
        if not req or not req.sort:
            return [self.DEFAULT_SORT]
        sort_fields = req.sort.split(',')
        for field in sort_fields:
            if field not in self.SORT_FIELDS:
                raise SuperdeskApiError.badRequestError('Invalid sort field %s' % field)
        return sort_fields
